SystemOrganization addCategory: #'Fuel-Core'!
SystemOrganization addCategory: #'Fuel-Core-Clusters'!
SystemOrganization addCategory: #'Fuel-Core-Clusters-Optionals'!
SystemOrganization addCategory: #'Fuel-Core-Mappers'!
SystemOrganization addCategory: #'Fuel-Core-Errors'!
SystemOrganization addCategory: #'Fuel-Collections'!
SystemOrganization addCategory: #'Fuel-Streams'!
SystemOrganization addCategory: #'Fuel-Utilities'!
SystemOrganization addCategory: #'Fuel-Core-Clusters-InternalMetalevel'!

!ByteSymbol methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:15'!
fuelAccept: aVisitor

	^aVisitor visitSymbol: self! !

!Set methodsFor: '*Fuel' stamp: 'MartinDias 9/9/2011 14:46'!
addIfNotPresent: anObject ifPresentDo: aBlock
	"Include anObject as one of the receiver's elements and then value aBlock, but only if there
	is no such element already. Anwser anObject."

	| index |
	index := self scanFor: anObject.
	(array at: index) 
		ifNil: [self atNewIndex: index put: anObject asSetElement]
		ifNotNil: [ aBlock value ].
	^ anObject

	"Sets in Pharo 1.2 can't' contain nil, so this implementation is wrong in that version. This is the right one (in Pharo 1.2):

	| index |
	anObject ifNil: [self error: 'Sets cannot meaningfully contain nil as an element'].
	index := self findElementOrNil: anObject.
	(array at: index) 
		ifNil: [ self atNewIndex: index put: anObject ]
		ifNotNil: [ aBlock value ].
	^ anObject"! !

!CompiledMethod methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:13'!
fuelAccept: aVisitor

	^aVisitor visitCompiledMethod: self! !

!CompiledMethod methodsFor: '*Fuel' stamp: 'MartinDias 10/22/2010 12:56'!
isNamedPrimitive 

	^ self primitive = 117! !

!Character methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:17'!
fuelAccept: aVisitor

	^aVisitor visitCharacter: self! !

!ByteArray methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:12'!
fuelAccept: aVisitor

	^aVisitor visitBytesObject: self! !

!TimeStamp methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 15:39'!
fuelAccept: aVisitor

	^ aVisitor visitFixedObject: self
! !

Error subclass: #FLClassNotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Errors'!

!FLClassNotFound commentStamp: 'MarianoMartinezPeck 6/5/2011 12:31' prior: 0!
FLClassNotFound is an error that happens when trying to materialize an instance whose class was removed between serialization and materialization time.!

!WriteStream methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 8/25/2011 12:12'!
nextByteStringPut: s 
	"Append the string, s, to the receiver.  Only used by DataStream.  Max size of 64*256*256*256."

	^ self nextStringPut: s! !

!WriteStream methodsFor: '*Fuel' stamp: 'MartinDias 8/13/2011 14:28'!
nextBytesPutAll: aCollection
	"Append the bytes of aCollection to the sequence of bytes accessible 
	by the receiver. Answer aCollection."
 
 	| newEnd |
 	collection class instSpec == aCollection class instSpec ifFalse:
 		[^ super nextPutAll: aCollection ].
 
 	newEnd := position + aCollection size.
 	newEnd > writeLimit ifTrue:
 		[self growTo: newEnd + 10].
 
 	collection replaceFrom: position+1 to: newEnd  with: aCollection startingAt: 1.
 	position := newEnd.
	
	^ aCollection! !

!StandardFileStream methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/6/2011 12:42'!
fuelNextWordsInto: aWordObject
	"This method is the same as nextWordsInto: but the restoreEndianness is only done if needed"
	self next: aWordObject basicSize into: aWordObject startingAt: 1.
	^ aWordObject! !

!Behavior methodsFor: '*Fuel' stamp: 'MartinDias 6/13/2011 01:04'!
fuelIgnoredInstanceVariableNames
	"Indicates which variables have to be ignored during serialization."

	^#()! !

!Behavior methodsFor: '*Fuel' stamp: 'MartinDias 6/12/2011 23:28'!
instanceVariableNames

	| variables |
	variables := Array new: self instSize.
	self instVarNamesAndOffsetsDo: [ :name :index | variables at: index put: name ].
	^variables! !

!UndefinedObject methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:42'!
fuelAccept: aVisitor

	^aVisitor visitUndefinedObject: self! !

!Association methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/9/2011 18:52'!
fuelAccept: aVisitor

	^ aVisitor visitAssociation: self
! !

!PositionableStream methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/6/2011 17:02'!
fuelNextWordsInto: aWordObject 
	"This method is the same as nextWordsInto: but the restoreEndianness is only done if needed"
	| blt pos source byteSize |

	byteSize := aWordObject byteSize.
	"is the test on collection basicSize \\ 4 necessary?"
	
	((self position bitAnd: 3) = 0 and: [ (collection basicSize bitAnd: 3) = 0])
		ifTrue: [source := collection.
			pos := self position.
			self skip: byteSize]
		ifFalse: ["forced to copy it into a buffer"
			source := self next: byteSize.
			pos := 0].

	"Now use BitBlt to copy the bytes to the bitmap."
	blt := (BitBlt current
				toForm: (Form new hackBits: aWordObject))
				sourceForm: (Form new hackBits: source).
	blt combinationRule: Form over. "store"
	blt sourceX: 0;
		 sourceY: pos // 4;
		 height: byteSize // 4;
		 width: 4.
	blt destX: 0;
		 destY: 0.
	blt copyBits.

	^ aWordObject 	
! !

!Collection methodsFor: '*Fuel' stamp: 'MartinDias 5/19/2011 23:41'!
addIfNotPresent: anObject ifPresentDo: aBlock
	"Include anObject as one of the receiver's elements and then value aBlock, but only if there
	is no such element already. Anwser anObject."

	(self includes: anObject) 
		ifFalse: [ self add: anObject ]
		ifTrue: [ aBlock value ].
	^ anObject! !

!SystemDictionary methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:40'!
fuelAccept: aVisitor

	^aVisitor visitSystemDictionary: self! !

!Boolean methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:35'!
fuelAccept: aVisitor

	^aVisitor visitBoolean: self! !

!Rectangle methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/8/2011 22:55'!
fuelAccept: aVisitor

	^ aVisitor visitRectangle: self
! !

!Trait class methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 6/29/2011 14:38'!
fuelNew
	^ super new! !

!Trait methodsFor: '*Fuel' stamp: 'MartinDias 7/1/2011 04:32'!
basicEnvironment

	^self environment ! !

!Trait methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:33'!
fuelAccept: aVisitor

	^aVisitor visitTrait: self! !

!Quadrangle methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/8/2011 22:55'!
fuelAccept: aVisitor

	^ aVisitor visitFixedObject: self
! !

!Float methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/6/2011 18:15'!
fuelAccept: aVisitor

	^aVisitor visitFloat: self! !

!Metaclass methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:26'!
fuelAccept: aVisitor

	^aVisitor visitMetaclass: self
! !

!ContextPart class methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 5/22/2011 23:44'!
newFromFrameSize: aFrameSize
	
	^ super basicNew: aFrameSize! !

Stream subclass: #FLBufferedWriteStream
	instanceVariableNames: 'stream buffer position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Streams'!

!FLBufferedWriteStream commentStamp: 'MarianoMartinezPeck 6/5/2011 12:41' prior: 0!
FLBufferedWriteStream is a buffered write stream we use for Fuel serialization. Instead of directly using the stream provided to FLSerializer at creation time by the user, we create an instance of FLBufferedWriteStream for that stream.

MultiByteFileStream has no real buffer and goes to disk too frequently. With FLBufferedWriteStream we keep stuff in a cache and only go to disk when this is full.

The way of using it is jut FLBufferedWriteStream on: aWriteStream. For example:

FLBufferedWriteStream on: (FileDirectory default forceNewFileNamed:  'TestFile') binary

With the message #sizeBuffer: you can set the size of the buffer.

Make sure to always send #flush or #close when you're done, otherwise the last buffer might not yet have been written.
!

!FLBufferedWriteStream class methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
on: writeStream
	^ self basicNew
		on: writeStream;
		yourself! !

!FLBufferedWriteStream class methodsFor: 'convenience' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
on: writeStream do: block
	"Execute block with as argument a ZnBufferedWriteStream on writeStream,
	making sure #flush is called at the end."
	
	| bufferedWriteStream |
	bufferedWriteStream := self on: writeStream.
	block value: bufferedWriteStream.
	bufferedWriteStream flush! !

!FLBufferedWriteStream methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
buffer
	buffer isNil 
		ifTrue: [ self sizeBuffer: self defaultBufferSize ].
	^ buffer! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
bufferFreeSize
	^ self bufferSize - position! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
bufferSize
	^ buffer isNil 
		ifTrue: [ self defaultBufferSize ]
		ifFalse: [ buffer size ]! !

!FLBufferedWriteStream methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
close
	self flushBuffer.
	stream close! !

!FLBufferedWriteStream methodsFor: 'private' stamp: 'MarianoMartinezPeck 9/8/2011 15:35'!
copyWordObjectToBuffer: aWordObject

	| blt |
	
	blt := (BitBlt current
				toForm: (Form new hackBits: self buffer))
				sourceForm: (Form new hackBits: aWordObject).
	blt combinationRule: Form over. "store"
	blt sourceX: 0;
		 sourceY: 0;
		 height: aWordObject byteSize // 4;
		 width: 4.
	blt destX: 0;
		 destY:  position // 4 .
	blt copyBits.

! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
defaultBufferSize
	^ 8192 "2 raisedTo: 13  "
	

! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
flush
	self flushBuffer.
	stream flush! !

!FLBufferedWriteStream methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
flushBuffer
	position = 0 ifTrue: [ ^ self ].
	position = self bufferSize
		ifTrue: [
			stream nextPutAll: buffer ]
		ifFalse: [
			(stream respondsTo: #next:putAll:startingAt:)
				ifTrue: [ stream next: position putAll: buffer startingAt: 1 ]
				ifFalse: [ stream nextPutAll: (buffer copyFrom: 1 to: position) ] ].
	position := 0! !

!FLBufferedWriteStream methodsFor: 'private' stamp: 'MarianoMartinezPeck 9/8/2011 17:46'!
flushBufferIfFull
	position = self bufferSize 
		ifTrue: [ self flushBuffer ]
! !

!FLBufferedWriteStream methodsFor: 'reading' stamp: 'MarianoMartinezPeck 9/9/2011 17:15'!
int32
	"Read a 32-bit signed integer from the next 4 bytes"
	| s |
	s := 0.
	1 to: 4 do: [:i | s := (s bitShift: 8) + self next].
	(s bitAnd: 16r80000000) = 0
		ifTrue: [^ s]
		ifFalse: [^ -1 - s bitInvert32]! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 9/9/2011 17:14'!
int32: int32
	"Write a signed integer to the next 4 bytes"
	| pos |
	self flushBufferIfFull.
	pos := int32 < 0
		ifTrue: [(0-int32) bitInvert32 + 1]
		ifFalse: [int32].
	1 to: 4 do: [:i | self nextPut: (pos digitAt: 5-i)].
	^ int32! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
nextNumber: n put: v 
	"Append to the receiver the argument, v, which is a positive 
	SmallInteger or a LargePositiveInteger, as the next n bytes.
	Possibly pad with leading zeros."
	self flushBufferIfFull.
	1 to: n do: [:i | self nextPut: (v digitAt: n+1-i)].
	^ v
! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
nextPut: object
	self flushBufferIfFull.
	position := position + 1.
	self buffer at: position put: object! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
nextPutAll: collection
	self flushBufferIfFull.
	collection size <= self bufferFreeSize
		ifTrue: [
			self buffer replaceFrom: position + 1 to: position + collection size with: collection.
			position := position + collection size ]
		ifFalse: [
			self flushBuffer.
			collection size > (self bufferSize / 2)
				ifTrue: [ stream nextPutAll: collection ]
				ifFalse: [ self nextPutAll: collection ] ]
! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 8/25/2011 13:18'!
nextStringPut: s 
	"Append the string, s, to the receiver.  Only used by DataStream.  Max size of 64*256*256*256."

	| length |
	self flushBufferIfFull.
	(length := s size) < 192
		ifTrue: [self nextPut: length]
		ifFalse: 
			[self nextPut: (length digitAt: 4)+192.
			self nextPut: (length digitAt: 3).
			self nextPut: (length digitAt: 2).
			self nextPut: (length digitAt: 1)].
	self nextPutAll: s.
	^s! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
nextWordPut: aWord 
	"Append to the receiver an Integer as the next two bytes."
	self flushBufferIfFull.
	self nextPut: ((aWord bitShift: -8) bitAnd: 255).
	self nextPut: (aWord bitAnd: 255).
	^aWord! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 9/8/2011 17:43'!
nextWordsPut: aWordObject
	| byteSize |

	byteSize := aWordObject byteSize.

	"Ensure we are at bigger than the words added, with size next power-of-two"
	byteSize > self bufferSize ifTrue: [ self sizeBuffer: 1 << ( byteSize highBit )].
	
	"BitBlt needs word-aligned access of object. Flushing the buffer is a very good idea because after the position will be zero, which is word aligned.  Word objects always have a full number of words of data to write. (otherwise they'd be variableByte objects or something :P) So as long as the size you write per instance also takes N words, the position will always be aligned (except when writing the first object of a cluster) . After that, we flush when an object larger than current buffer is encountered (statistically rare), or if the buffer is full (which we do anyways).
	Finally, we also need enough free space in the buffer"
	((position bitAnd: 3) = 0 and: [byteSize < self bufferFreeSize])
      	ifFalse: [self flushBuffer].

	self copyWordObjectToBuffer: aWordObject.
	position := position +  byteSize.! !

!FLBufferedWriteStream methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
on: writeStream
	stream := writeStream.
	position := 0! !

!FLBufferedWriteStream methodsFor: 'printing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/8/2011 15:55'!
sizeBuffer: size
    buffer ifNotNil: [self flushBuffer].
    buffer := (stream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]) new: size.
! !

!FLBufferedWriteStream methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 23:32'!
space
	self nextPut: Character space! !

!FLBufferedWriteStream methodsFor: 'reading' stamp: 'MarianoMartinezPeck 9/9/2011 17:16'!
uint32
	"Read a 32-bit signed integer from the next 4 bytes"
	| s |
	s := 0.
	1 to: 4 do: [:i | s := (s bitShift: 8) + self next].
	^ s! !

!FLBufferedWriteStream methodsFor: 'writing' stamp: 'MarianoMartinezPeck 9/9/2011 17:13'!
uint32: int32
	"Write a signed integer to the next 4 bytes"
	self flushBufferIfFull.
	1 to: 4 do: [:i | self nextPut: (int32 digitAt: 5-i)].
	^ int32! !

!Stream methodsFor: '*Fuel' stamp: 'MartinDias 8/13/2011 14:29'!
nextBytesPutAll: aCollection
	"Append the bytes of aCollection to the sequence of bytes accessible 
	by the receiver. Answer aCollection."

	^self nextPutAll: aCollection! !

!ClassTrait methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:23'!
fuelAccept: aVisitor

	^aVisitor visitClassTrait: self
! !

!Bitmap methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/8/2011 17:45'!
fuelAccept: aVisitor

	^aVisitor visitBitmap: self! !

!Array methodsFor: '*Fuel' stamp: 'MartinDias 8/15/2011 22:52'!
fuelAccept: aVisitor

	^aVisitor visitVariableObject: self! !

!MethodContext methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:28'!
fuelAccept: aVisitor

	^aVisitor visitMethodContext: self! !

!MethodContext methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 5/22/2011 22:11'!
method: aMethod

	method := aMethod ! !

!Time methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 10:32'!
fuelAccept: aVisitor

	^ aVisitor visitTime: self
! !

!CharacterBlock methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/8/2011 22:56'!
fuelAccept: aVisitor

	^ aVisitor visitFixedObject: self
! !

!Duration methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 10:31'!
fuelAccept: aVisitor

	^ aVisitor visitDuration: self
! !

!SmallInteger methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:31'!
fuelAccept: aVisitor

	^aVisitor visitSmallInteger: self! !

!Date class methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 10:28'!
fuelFromjulianDayNumber: aJulianDayNumber

	^ self starting: (DateAndTime julianDayNumber: aJulianDayNumber) midnight 
duration: (Duration days: 1)! !

!Date methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 10:31'!
fuelAccept: aVisitor

	^ aVisitor visitDate: self
! !

!HashedCollection methodsFor: '*Fuel' stamp: 'MartinDias 6/13/2011 02:43'!
fuelAfterMaterialization

	self rehash! !

!Point methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/8/2011 21:42'!
fuelAccept: aVisitor

	^ aVisitor visitPoint: self
! !

!DateAndTime class methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 15:14'!
fuelFrom: julianDayNumber nanoSecond: nanoSeconds seconds: seconds offset: anOffset
	^ self basicNew
		fuelSet: julianDayNumber nanoSecond: nanoSeconds seconds: seconds offset: anOffset! !

!DateAndTime methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 15:17'!
fuelAccept: aVisitor

	^ aVisitor visitDateAndTime: self
! !

!DateAndTime methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/2/2011 15:15'!
fuelSet: julianDayNumber nanoSecond: nanoSeconds seconds: numberOfSeconds offset: anOffset
	jdn := julianDayNumber.
	nanos := nanoSeconds.
	seconds := numberOfSeconds.
	offset := anOffset.! !

!LargePositiveInteger methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:39'!
fuelAccept: aVisitor

	^aVisitor visitLargeInteger: self! !

!ByteString methodsFor: '*Fuel' stamp: 'MartinDias 8/15/2011 17:20'!
fuelAccept: aVisitor

	^aVisitor visitByteString: self! !

!MethodDictionary methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 9/17/2011 14:36'!
fuelAccept: aVisitor

	^ aVisitor visitMethodDictionary: self
! !

!TPureBehavior methodsFor: '*Fuel' stamp: 'MartinDias 8/29/2011 04:06'!
fuelInitializeTraitComposition

	self traitComposition traits do: [:each | each addUser: self]! !

!ClassDescription methodsFor: '*Fuel' stamp: 'MartinDias 7/30/2011 19:16'!
basicInstVarNames
	"Answer nil or an Array of the receiver's instance variable names."

	^instanceVariables! !

!ClassDescription methodsFor: '*Fuel' stamp: 'MartinDias 7/30/2011 19:23'!
basicInstVarNames: anArray

	instanceVariables := anArray! !

!ClassDescription methodsFor: '*Fuel' stamp: 'MartinDias 7/27/2011 22:33'!
fuelInstVarNamesAndOffsetsDo: aBinaryBlock
	"This is part of the interface between the compiler and a class's instance or field names.
	 The class should enumerate aBinaryBlock with the instance variable name strings and
	 their integer offsets.  The order is important. Names evaluated later will override the
	 same names occurring earlier."

	| superInstSize |
	(superInstSize := superclass notNil ifTrue: [superclass instSize] ifFalse: [0]) > 0 ifTrue:
		[superclass fuelInstVarNamesAndOffsetsDo: aBinaryBlock].
	1 to: self instSize - superInstSize do:
		[:i| aBinaryBlock value: (instanceVariables at: i) value: i + superInstSize]! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 7/30/2011 18:23'!
basicClassPool
	"Answer nil or the dictionary of class variables."

	^classPool! !

!Class methodsFor: '*Fuel' stamp: 'MarianoMartinezPeck 6/29/2011 11:04'!
basicEnvironment

	^ environment! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 7/30/2011 18:23'!
basicSharedPools
	"Answer nil or a Set of the pool dictionaries declared in the receiver."

	^sharedPools! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 5/14/2011 21:44'!
basicSubclasses

	^ subclasses! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 5/14/2011 21:45'!
basicSubclasses: anArray

	subclasses := anArray! !

!Class methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:22'!
fuelAccept: aVisitor

	^aVisitor visitClass: self! !

Object subclass: #FLAnalysis
	instanceVariableNames: 'objectStack firstMapper clusterization root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLAnalysis commentStamp: 'MartinDias 8/29/2011 19:15' prior: 0!
I am responsible of traversing the graph of references starting from a root object. I will produce a clusterization which the serializer will store on a stream.

An example of use is:

	(FLAnalyzer newDefault 
		clusterizationOf: (Array with: 1@2 with: 3@4))
		clusters.
!

!FLAnalysis class methodsFor: 'instance creation' stamp: 'MartinDias 9/9/2011 22:13'!
newWith: aMapper root: anObject

	^self basicNew
		initializeWith: aMapper root: anObject;
		yourself.! !

!FLAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 8/7/2011 20:18'!
clusterFor: anObject

	^firstMapper chainedClusterFor: anObject! !

!FLAnalysis methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 21:47'!
clusterization

	^clusterization! !

!FLAnalysis methodsFor: 'initialize-release' stamp: 'MartinDias 9/9/2011 22:03'!
initializeWith: aMapper root: anObject

	super initialize.
	firstMapper := aMapper.
	root := anObject.
	objectStack := FLSimpleStack new.
	clusterization := FLClusterization new.
! !

!FLAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:22'!
mapAndTrace: anObject
	"Map an object to its cluster. Trace its references."

	| aCluster |
	aCluster := self clusterFor: anObject.
	clusterization add: aCluster traceWith: self.
	aCluster add: anObject traceWith: self.
! !

!FLAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 9/10/2011 18:08'!
run

	objectStack push: root.
	[ objectStack isEmpty ] 
		whileFalse: [ self mapAndTrace: objectStack pop ].! !

!FLAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:11'!
trace: anObject

	objectStack push: anObject! !

Object subclass: #FLAnalyzer
	instanceVariableNames: 'internalClasses pluggableSubstitutions globalMappersFactory analysisFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLAnalyzer commentStamp: 'MartinDias 8/29/2011 19:15' prior: 0!
I am responsible of traversing the graph of references starting from a root object. I will produce a clusterization which the serializer will store on a stream.

An example of use is:

	(FLAnalyzer newDefault 
		clusterizationOf: (Array with: 1@2 with: 3@4))
		clusters.
!

!FLAnalyzer class methodsFor: 'instance creation' stamp: 'MartinDias 9/9/2011 21:25'!
newDefault
	"Returns an instance with default configuration."

	^self new! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 9/13/2011 22:24'!
allGlobalMappers

	^{FLGlobalAssociationMapper. FLClassPoolAssociationMapper. FLGlobalMapper. FLGlobalBehaviorMapper.} 
		collect: [:aMapperClass | aMapperClass new internalObjects: internalClasses; yourself ]! !

!FLAnalyzer methodsFor: 'analyzing' stamp: 'MartinDias 9/14/2011 00:15'!
analysisFor: anObject

	^analysisFactory value: anObject! !

!FLAnalyzer methodsFor: 'configuring' stamp: 'MartinDias 9/11/2011 12:33'!
considerGlobalMinimaly

	globalMappersFactory := [self minimalGlobalsMappers]! !

!FLAnalyzer methodsFor: 'configuring' stamp: 'MartinDias 9/11/2011 11:19'!
considerGlobalWhenPossible

	globalMappersFactory := [self allGlobalMappers]! !

!FLAnalyzer methodsFor: 'configuring' stamp: 'MartinDias 9/11/2011 11:26'!
considerInternalAll: classes

	internalClasses addAll: classes.
	self considerGlobalWhenPossible.! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 9/10/2011 17:57'!
defaultMapper

	^FLDefaultMapper new! !

!FLAnalyzer methodsFor: 'analyzing' stamp: 'MartinDias 9/10/2011 18:10'!
firstInMapperChain

	^self mappers
		reduceRight: [:left :right | left next: right ];
		first
! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 9/11/2011 11:22'!
globalMappers
	
	^globalMappersFactory value! !

!FLAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 9/13/2011 19:25'!
initialize

	super initialize.
	self setDefaultAnalysis.
	self considerGlobalMinimaly.
	pluggableSubstitutions := OrderedCollection new.
	internalClasses := OrderedCollection new.
! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 9/10/2011 17:59'!
mappers

	| mappers |
	mappers := OrderedCollection new.
	mappers addAll: self pluggableSubstitutionMappers.
	mappers addAll: self globalMappers.
	mappers add: self defaultMapper.
	^mappers! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 9/11/2011 12:33'!
minimalGlobalsMappers

	^Array with: FLLightGlobalMapper new! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 9/10/2011 17:55'!
pluggableSubstitutionMappers

	^pluggableSubstitutions collect: [:aLink |
		FLPluggableSubstitutionMapper 
			when: aLink key
			substituteBy: aLink value]! !

!FLAnalyzer methodsFor: 'protected' stamp: 'MartinDias 9/14/2011 00:24'!
setDefaultAnalysis

	analysisFactory := [:anObject | 
		(FLAnalysis 
			newWith: self firstInMapperChain 
			root: anObject)
			run;
			yourself ]! !

!FLAnalyzer methodsFor: 'configuring' stamp: 'MartinDias 9/10/2011 17:04'!
when: aCondition substituteBy: aFactory

	pluggableSubstitutions add: aCondition -> aFactory! !

Object subclass: #FLCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLCluster commentStamp: 'MartinDias 8/29/2011 19:20' prior: 0!
I represent a cluster of objects grouped by some specific similarity. I know how to serialize and materialize them all together.!

!FLCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/29/2011 00:46'!
add: anObject traceWith: anAnalyzer
	"Add an object to the cluster and trace references."

	self subclassResponsibility! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 23:57'!
afterMaterializationStepWith: aMaterializer
! !

!FLCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/29/2011 00:54'!
clusterBucketIn: aClusterization
	"Answer in which clusters bucket place this cluster. This defines the sort order in which the clusters will be ordered. See FLAnalyzer >> clusteredObjects for more information."

	self subclassResponsibility ! !

!FLCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/29/2011 00:54'!
clusterReferencesDo: aBlock
	"Evaluate a block with each object referenced by the cluster"! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:37'!
materializeInstancesStepWith: aMaterializer

	self subclassResponsibility ! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 19:24'!
materializePostInstancesStepWith: aMaterializer

! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:52'!
materializeReferencesStepWith: aMaterializer
! !

!FLCluster methodsFor: 'accessing' stamp: 'MartinDias 8/29/2011 00:50'!
objects
	"Answer the objects that this cluster groups."

	self subclassResponsibility! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/29/2011 01:23'!
registerIndexesOn: aDictionary
	"Maps objects with indexes in the dictionary."! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:35'!
serializeInstancesStepWith: aSerializer

	self subclassResponsibility ! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 19:24'!
serializePostInstancesStepWith: aSerializer
! !

!FLCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:36'!
serializeReferencesStepWith: aSerializer
! !

FLCluster subclass: #FLIteratingCluster
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLIteratingCluster commentStamp: '<historical>' prior: 0!
I am a template class whose algorithm for serialize a collection of objects consists on delegate the serialization of each individual object to the subclass.!

FLIteratingCluster subclass: #FLAssociationCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLAssociationCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 9/9/2011 18:14'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket! !

!FLAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 18:17'!
materializeInstanceWith: aMaterializer
	"Since in Pharo #@ is associated with a special bytecode, it is faster than doing  Point basicNew"
	^ Association basicNew
	! !

!FLAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 18:17'!
materializeReferencesStepWith: aMaterializer

	objects do: [ :anAssociation | 
		anAssociation key: aMaterializer nextEncodedReference value: aMaterializer nextEncodedReference.	
	 ]! !

!FLAssociationCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 9/9/2011 18:17'!
referencesOf: anAssociation do: aBlock

	aBlock value: anAssociation key.
	aBlock value: anAssociation value.
	! !

!FLAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/18/2011 13:58'!
serializeInstances: aSerializer
	! !

!FLAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 18:19'!
serializeReferencesStepWith: aSerializer

	objects do: [ :anAssociation | 
		aSerializer encodeReferenceTo: anAssociation key.
		aSerializer encodeReferenceTo: anAssociation value.
	]! !

FLIteratingCluster subclass: #FLBehaviorCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-InternalMetalevel'!

!FLBehaviorCluster commentStamp: 'MartinDias 7/30/2011 20:31' prior: 0!
I have the common behavior for serializing class and trait behaviors in full detail.!

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 23:57'!
afterMaterializationStepWith: aMaterializer

	objects do:	[ :aClassOrTrait | aClassOrTrait fuelInitializeTraitComposition ]! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 19:45'!
materializePostInstance: aClassOrTrait with: aMaterializer

	self subclassResponsibility! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 19:45'!
materializePostInstancesStepWith: aMaterializer

	objects do:	[ :aClassOrTrait | self materializePostInstance: aClassOrTrait with: aMaterializer ]! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 23:27'!
materializeReferencesOf: aClassOrTrait with: aMaterializer

	aClassOrTrait methodDictionary: aMaterializer nextEncodedReference.
	aClassOrTrait organization: aMaterializer nextEncodedReference.
	aClassOrTrait basicLocalSelectors: aMaterializer nextEncodedReference.
	aClassOrTrait traitComposition: aMaterializer nextEncodedReference.! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:52'!
materializeReferencesStepWith: aMaterializer

	objects do:	[ :aClassOrTrait | self materializeReferencesOf: aClassOrTrait with: aMaterializer ]! !

!FLBehaviorCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/30/2011 20:17'!
referencesOf: aClassOrTrait do: aBlock

	aBlock value: aClassOrTrait methodDictionary.
	aBlock value: aClassOrTrait organization.
	aBlock value: aClassOrTrait traitComposition.
	aBlock value: aClassOrTrait basicLocalSelectors.! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 19:45'!
serializePostInstance: aClassOrTrait with: aSerializer

	self subclassResponsibility! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 19:45'!
serializePostInstancesStepWith: aSerializer  

	objects do:	[ :aClassOrTrait | self serializePostInstance: aClassOrTrait with: aSerializer ]! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 23:27'!
serializeReferencesOf: aClassOrTrait with: aSerializer

	aSerializer encodeReferenceTo: aClassOrTrait methodDictionary.
	aSerializer encodeReferenceTo: aClassOrTrait organization. 
	aSerializer encodeReferenceTo: aClassOrTrait basicLocalSelectors.
	aSerializer encodeReferenceTo: aClassOrTrait traitComposition.! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:36'!
serializeReferencesStepWith: aSerializer

	objects do:	[ :aClassOrTrait | self serializeReferencesOf: aClassOrTrait with: aSerializer ]! !

FLBehaviorCluster subclass: #FLClassCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-InternalMetalevel'!

!FLClassCluster commentStamp: 'MartinDias 7/30/2011 20:31' prior: 0!
I am a cluster for storing classes in full detail.!

!FLClassCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 00:27'!
clusterBucketIn: aClusterization

	^aClusterization metalevelInstanceSideBucket! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 19:47'!
materializeInstanceWith: aMaterializer

	| aMetaclass |
	aMetaclass := aMaterializer nextEncodedReference.
	^aMetaclass new! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/4/2011 11:47'!
materializePostInstance: aClass with: aMaterializer

	| superclass category name instanceVariableNames format |
	superclass := aMaterializer nextEncodedReference.
	category := aMaterializer nextEncodedReference.
	name := aMaterializer nextEncodedReference.
	format := aMaterializer nextEncodedReference.
	instanceVariableNames := aMaterializer stream nextString subStrings: ' '.

	aClass 
		superclass: superclass;
		setFormat: format;
		setName: name;
		basicInstVarNames: instanceVariableNames;
		basicCategory: category; 
		methodDict: MethodDictionary new.! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/30/2011 20:20'!
materializeReferencesOf: aClass with: aMaterializer

	super materializeReferencesOf: aClass with: aMaterializer.

	aClass classPool: aMaterializer nextEncodedReference.
	aClass sharedPools: aMaterializer nextEncodedReference.
	aClass basicSubclasses: aMaterializer nextEncodedReference. 
	aClass environment: aMaterializer nextEncodedReference.
	
	Class class instSize + 1 to: aClass class instSize 
		do: [ :index | aClass instVarAt: index put: aMaterializer nextEncodedReference ]! !

!FLClassCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/30/2011 20:18'!
referencesOf: aClass do: aBlock

	super referencesOf: aClass do: aBlock.

	aBlock value: aClass classSide.
	aBlock value: aClass superclass.
	aBlock value: aClass basicClassPool.
	aBlock value: aClass basicSharedPools. 
	aBlock value: aClass basicSubclasses.
	aBlock value: aClass basicCategory.
	aBlock value: aClass name.
	aBlock value: aClass format.	
	aBlock value: aClass basicEnvironment.
	
	Class class instSize + 1 to: aClass class instSize 
		do: [ :index | aBlock value: (aClass instVarAt: index) ]! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 19:48'!
serializeInstance: aClass with: aSerializer

	aSerializer encodeReferenceTo: aClass classSide! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 19:47'!
serializePostInstance: aClass with: aSerializer

	aSerializer encodeReferenceTo: aClass superclass.
	aSerializer encodeReferenceTo: aClass basicCategory.
	aSerializer encodeReferenceTo: aClass name.
	aSerializer encodeReferenceTo: aClass format.	
	aSerializer stream nextStringPut: aClass instanceVariablesString.
	self flag: #todo. "The array of variable names represents a circularity problem between clusters: if we encode the reference to such array, then the cluster for arrays should be before of Class cluster. But this would disable us to serialize an array of internal classes."
! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/30/2011 20:18'!
serializeReferencesOf: aClass with: aSerializer

	super serializeReferencesOf: aClass with: aSerializer.

	aSerializer encodeReferenceTo: aClass basicClassPool. 
	aSerializer encodeReferenceTo: aClass basicSharedPools. 
	aSerializer encodeReferenceTo: aClass basicSubclasses.
	aSerializer encodeReferenceTo: aClass basicEnvironment.
 
	Class class instSize + 1 to: aClass class instSize 
		do: [ :index | aSerializer encodeReferenceTo: (aClass instVarAt: index) ]! !

FLBehaviorCluster subclass: #FLClassTraitCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-InternalMetalevel'!

!FLClassTraitCluster commentStamp: 'MartinDias 7/30/2011 20:30' prior: 0!
I am a cluster for storing class traits in full detail.!

!FLClassTraitCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 00:27'!
clusterBucketIn: aClusterization

	^aClusterization metalevelClassSideBucket! !

!FLClassTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/16/2011 23:35'!
materializeInstanceWith: aMaterializer

	^ClassTrait basicNew! !

!FLClassTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/16/2011 23:57'!
materializePostInstance: aClassTrait with: aMaterializer

	aClassTrait 
		initialize.! !

!FLClassTraitCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/18/2011 14:54'!
serializeInstances: aSerializer! !

!FLClassTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 20:04'!
serializePostInstance: aClassTrait with: aSerializer
! !

FLBehaviorCluster subclass: #FLMetaclassCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-InternalMetalevel'!

!FLMetaclassCluster commentStamp: 'MartinDias 7/30/2011 20:30' prior: 0!
I am a cluster for storing metaclasses in full detail.!

!FLMetaclassCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 00:27'!
clusterBucketIn: aClusterization

	^aClusterization metalevelClassSideBucket! !

!FLMetaclassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 19:50'!
materializeInstanceWith: aMaterializer

	^Metaclass new! !

!FLMetaclassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 21:37'!
materializePostInstance: aMetaclass with: aMaterializer

	| superclass instanceVariableNames format |
	superclass := aMaterializer nextEncodedReference.
	instanceVariableNames := aMaterializer stream nextString subStrings: ' '.
	format := aMaterializer stream nextWord.

	aMetaclass
		superclass: superclass 
		methodDictionary: MethodDictionary new 
		format: format;
		basicInstVarNames: instanceVariableNames.! !

!FLMetaclassCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/29/2011 21:12'!
referencesOf: aMetaclass do: aBlock

	super referencesOf: aMetaclass do: aBlock.
	
	aBlock value: aMetaclass superclass.! !

!FLMetaclassCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/18/2011 14:54'!
serializeInstances: aSerializer! !

!FLMetaclassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 19:50'!
serializePostInstance: aMetaclass with: aSerializer

	aSerializer encodeReferenceTo: aMetaclass superclass.
	aSerializer stream nextStringPut: aMetaclass instanceVariablesString.
	aSerializer stream nextWordPut: aMetaclass format.
	! !

FLBehaviorCluster subclass: #FLTraitCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-InternalMetalevel'!

!FLTraitCluster commentStamp: 'MartinDias 7/30/2011 20:30' prior: 0!
I am a cluster for storing traits in full detail.!

!FLTraitCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 00:27'!
clusterBucketIn: aClusterization

	^aClusterization metalevelInstanceSideBucket! !

!FLTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/17/2011 00:09'!
materializeInstanceWith: aMaterializer

	^Trait basicNew! !

!FLTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/17/2011 00:13'!
materializePostInstance: aTrait with: aMaterializer

	| classTrait category name environment |
	classTrait := aMaterializer nextEncodedReference.
	category := aMaterializer nextEncodedReference.
	name := aMaterializer nextEncodedReference.
	environment := aMaterializer nextEncodedReference.
	
	aTrait 
		initialize;
		name: name;
		basicCategory: category asSymbol;
		classTrait: classTrait;
		environment: environment.
	
	classTrait baseTrait: aTrait.! !

!FLTraitCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/16/2011 23:48'!
referencesOf: aTrait do: aBlock

	super referencesOf: aTrait do: aBlock.

	aBlock value: aTrait classSide.
	aBlock value: aTrait basicEnvironment.
	aBlock value: aTrait basicCategory.
	aBlock value: aTrait name.
! !

!FLTraitCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/18/2011 14:53'!
serializeInstances: aSerializer! !

!FLTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/17/2011 00:08'!
serializePostInstance: aTrait with: aSerializer

	aSerializer encodeReferenceTo: aTrait classSide.
	aSerializer encodeReferenceTo: aTrait basicCategory.
	aSerializer encodeReferenceTo: aTrait name.
	aSerializer encodeReferenceTo: aTrait basicEnvironment.
! !

FLIteratingCluster subclass: #FLClassPoolAssociationCluster
	instanceVariableNames: 'ownerClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLClassPoolAssociationCluster commentStamp: 'MartinDias 5/30/2011 01:17' prior: 0!
I am a cluster for class variable associations, i.e. bindings that belong to the class pool of a Class included on Smalltalk globals.!

!FLClassPoolAssociationCluster classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/20/2011 06:54'!
for: aClass

	^ self basicNew
		initialize;
		theClass: aClass;
		yourself.! !

!FLClassPoolAssociationCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:24'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket! !

!FLClassPoolAssociationCluster methodsFor: 'analyzing' stamp: 'MartinDias 6/5/2011 03:03'!
clusterReferencesDo: aBlock

	aBlock value: ownerClass! !

!FLClassPoolAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:34'!
materializeInstanceWith: aMaterializer

	| keyName |
	ownerClass := aMaterializer nextEncodedReference.
	keyName := aMaterializer stream nextString.
	^ ownerClass classPool associationAt: keyName asSymbol ! !

!FLClassPoolAssociationCluster methodsFor: 'printing' stamp: 'MartinDias 8/20/2011 23:09'!
printNameOn: aStream
	"Append to the argument, aStream, a sequence of characters that  
	identifies the receiver."
	
	super printNameOn: aStream.
	aStream nextPut: $[.
	self theClass printOn: aStream.
	aStream nextPut: $].
	! !

!FLClassPoolAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:28'!
serializeInstance: anAssociation with: aSerializer

	aSerializer encodeReferenceTo: ownerClass.
	aSerializer stream nextStringPut: anAssociation key! !

!FLClassPoolAssociationCluster methodsFor: 'accessing' stamp: 'MartinDias 6/5/2011 03:02'!
theClass

	^ ownerClass! !

!FLClassPoolAssociationCluster methodsFor: 'accessing' stamp: 'MartinDias 6/5/2011 03:02'!
theClass: aClass

	ownerClass := aClass! !

FLIteratingCluster subclass: #FLCompiledMethodCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLCompiledMethodCluster commentStamp: 'MartinDias 5/30/2011 01:17' prior: 0!
I am a cluster for CompiledMethod instances.!

!FLCompiledMethodCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 9/17/2011 19:31'!
clusterBucketIn: aClusterization

	^aClusterization primitivesBucket! !

!FLCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:34'!
materializeInstanceWith: aMaterializer

	| compiledMethod header bytecodesPlusTrailerSize aReadStream |

	aReadStream := aMaterializer stream.

	header := (aReadStream nextWord << 16) bitOr: aReadStream nextWord.
	bytecodesPlusTrailerSize := aReadStream nextWord.

	compiledMethod := CompiledMethod newMethod: bytecodesPlusTrailerSize header: header.

	compiledMethod initialPC to: compiledMethod size
		do: [ :index | compiledMethod at: index put: aReadStream next ].
		
	^ compiledMethod! !

!FLCompiledMethodCluster methodsFor: 'protected' stamp: 'MartinDias 6/3/2011 21:16'!
materializeLiteralsTo: compiledMethod with: aMaterializer

	1 to: compiledMethod numLiterals 
		do: [ :index | compiledMethod literalAt: index put: aMaterializer nextEncodedReference ]
! !

!FLCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:52'!
materializeReferencesStepWith: aMaterializer

	objects do: [ :aCompiledMethod | self materializeLiteralsTo: aCompiledMethod with: aMaterializer ]! !

!FLCompiledMethodCluster methodsFor: 'analyzing' stamp: 'MartinDias 3/26/2011 16:27'!
referencesOf: anObject do: aBlock

	1 to: anObject numLiterals
		do: [ :index | aBlock value: (anObject literalAt: index) ]! !

!FLCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:28'!
serializeInstance: aCompiledMethod with: aSerializer

	| header aWriteStream bytecodesPlusTrailerSize cmSize cmInitialPC |

	aWriteStream := aSerializer stream.

	header := aCompiledMethod header.
	cmSize := aCompiledMethod size.
	cmInitialPC := aCompiledMethod initialPC.
	bytecodesPlusTrailerSize := cmSize - cmInitialPC + 1.

	aWriteStream 
		nextWordPut: (header >> 16);
		nextWordPut: header;
		nextWordPut: bytecodesPlusTrailerSize.

	cmInitialPC to: cmSize
		do: [ :index | aWriteStream nextPut: (aCompiledMethod at: index) ].! !

!FLCompiledMethodCluster methodsFor: 'protected' stamp: 'MartinDias 6/3/2011 20:42'!
serializeLiteralsOf: aCompiledMethod with: aSerializer

	1 to: aCompiledMethod numLiterals 
		do: [ :index | aSerializer encodeReferenceTo: (aCompiledMethod literalAt: index) ]! !

!FLCompiledMethodCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:36'!
serializeReferencesStepWith: aSerializer

	objects do: [ :aCompiledMethod | self serializeLiteralsOf: aCompiledMethod with: aSerializer ]! !

FLCompiledMethodCluster subclass: #FLNamedPrimitiveMethodCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLNamedPrimitiveMethodCluster commentStamp: 'MartinDias 5/30/2011 01:17' prior: 0!
I am a cluster for CompiledMethods which are <primitive:module:> primitives.

For example, Integer >> bitAnd: should be serialized with me.!

!FLNamedPrimitiveMethodCluster methodsFor: 'protected' stamp: 'MartinDias 6/3/2011 21:16'!
materializeLiteralsTo: compiledMethod with: aMaterializer

	| module name |

	module := aMaterializer nextEncodedReference.
	name := aMaterializer nextEncodedReference.
	"The third element of the array is Obsolete. We can put whatever we want since it is not used anymore by the VM. It used to be a session ID. The forth element is the primitive index in a externalPrimitiveTable and it is IMPORTANT to put it in zero. Always. Otherwise there can be errors. Putting a zero means that the VM will not try to directly execute the primitive from the table and it will always try to load the module. It is the safer thing to do. For more details, read the method #primitiveExternalCall in VMMaker"
	compiledMethod literalAt: 1 put: (Array with: module with: name with: 0 with: 0).

	2 to: compiledMethod numLiterals 
		do: [ :index | compiledMethod literalAt: index put: aMaterializer nextEncodedReference ]
! !

!FLNamedPrimitiveMethodCluster methodsFor: 'analyzing' stamp: 'MartinDias 11/11/2010 18:31'!
referencesOf: anObject do: aBlock

	aBlock 
		value: (anObject literalAt: 1) first;
		value: (anObject literalAt: 1) second. 

	2 to: anObject numLiterals
		do: [ :index | aBlock value: (anObject literalAt: index) ]! !

!FLNamedPrimitiveMethodCluster methodsFor: 'protected' stamp: 'MartinDias 6/3/2011 20:42'!
serializeLiteralsOf: aCompiledMethod with: aSerializer

	| primitiveArguments |
	
	primitiveArguments := aCompiledMethod literalAt: 1.
	aSerializer 
		encodeReferenceTo: primitiveArguments first;
		encodeReferenceTo: primitiveArguments second.

	2 to: aCompiledMethod numLiterals 
		do: [ :index | aSerializer encodeReferenceTo: (aCompiledMethod literalAt: index) ]! !

!FLIteratingCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/29/2011 00:46'!
add: anObject traceWith: anAnalyzer
	"Add an object to the cluster and trace references."

	objects addIfNotPresent: anObject ifPresentDo: [ ^self ].

	self referencesOf: anObject do: [ :aChild | anAnalyzer trace: aChild ].! !

!FLIteratingCluster methodsFor: 'initialize-release' stamp: 'MartinDias 8/29/2011 00:47'!
initialize

	super initialize.
	objects := self newSerializingCollection.! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:34'!
materializeInstanceWith: aMaterializer
	"Hook method that each subclass should customize for its special way of materializing"

	^ self subclassResponsibility! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 20:20'!
materializeInstancesStepWith: aMaterializer

	| instanceCount |
	instanceCount := aMaterializer nextEncodedPositiveInteger.
	
	objects := self newMaterializingCollection: instanceCount.  
	
	instanceCount timesRepeat: [ 
		objects add: (self materializeInstanceWith: aMaterializer) ].! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/15/2011 12:36'!
newMaterializingCollection: instanceCount

	^(OrderedCollection new: instanceCount)
		resetTo: 1; "This avoids prematures #makeRoomAtLast. In fact, this has already been fixed in Pharo 1.4: http://code.google.com/p/pharo/issues/detail?id=1686"
		yourself.
! !

!FLIteratingCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/29/2011 00:48'!
newSerializingCollection
	"Answer a collection for the objects that correspond to this cluster."

	^IdentitySet new! !

!FLIteratingCluster methodsFor: 'accessing' stamp: 'MartinDias 8/29/2011 00:59'!
objects

	^objects! !

!FLIteratingCluster methodsFor: 'printing' stamp: 'MartinDias 8/29/2011 01:16'!
printNameOn: aStream

	super printOn: aStream! !

!FLIteratingCluster methodsFor: 'printing' stamp: 'MartinDias 8/29/2011 00:52'!
printOn: aStream

	self printNameOn: aStream.
	aStream nextPutAll: '->'.
	objects printElementsOn: aStream! !

!FLIteratingCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/29/2011 00:56'!
referencesOf: anObject do: aBlock
	"Evaluate a block with each object referenced by anObject"! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/29/2011 01:22'!
registerIndexesOn: aDictionary

	self objects do: [ :instance | aDictionary at: instance put: aDictionary size + 1 ].! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:28'!
serializeInstance: anObject with: aSerializer
	"Hook method that each subclass should customize for its special way of serializing"

	self subclassResponsibility ! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/18/2011 13:57'!
serializeInstances: aSerializer
	objects do: [ :instance | self serializeInstance: instance with: aSerializer ]! !

!FLIteratingCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/18/2011 13:57'!
serializeInstancesStepWith: aSerializer
	aSerializer encodePositiveInteger: objects size.
	self serializeInstances: aSerializer! !

FLIteratingCluster subclass: #FLMethodDictionaryCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLMethodDictionaryCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 9/17/2011 14:18'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket! !

!FLMethodDictionaryCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/17/2011 20:07'!
materializeInstanceWith: aMaterializer
	
	"It is a pity we cannot do new: with its correct size. But this is not possible because if we do so, there is no way to get back such size. Because MethodDIctionary #size answers those not nil, hence it will answer 0. And #basicSize answers a bigger number. Example:
	(MethodDictionary new: 11) size -> 0
	(MethodDictionary new: 11) basicSize -> 16
	
	So...we loose the 11, and we need that in the post references step. Hence, we read that from the stream at that moment. 
	"

	^ MethodDictionary new: aMaterializer nextEncodedPositiveInteger.
	! !

!FLMethodDictionaryCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/17/2011 19:32'!
materializeReferencesStepWith: aMaterializer
	| method size |
	objects do: [ :aMethodDictionary | 
		size := aMaterializer nextEncodedPositiveInteger.
		size timesRepeat: [
			method := aMaterializer nextEncodedReference.
			aMethodDictionary at: method selector put: method ].
		]! !

!FLMethodDictionaryCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 9/17/2011 14:21'!
referencesOf: aDictionary do: aBlock

	aDictionary do: [ :value | aBlock value: value ]! !

!FLMethodDictionaryCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/17/2011 20:04'!
serializeInstance: aMethodDictionary with: aSerializer
	aSerializer encodePositiveInteger: aMethodDictionary size.! !

!FLMethodDictionaryCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/17/2011 14:49'!
serializeReferencesStepWith: aSerializer

	objects do: [ :aDictionary | 
		aSerializer encodePositiveInteger: aDictionary size.
		aDictionary do: [ :value | aSerializer encodeReferenceTo: value ].
	].
! !

FLIteratingCluster subclass: #FLObjectCluster
	instanceVariableNames: 'theClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLObjectCluster commentStamp: 'MartinDias 8/29/2011 19:20' prior: 0!
I implement a generic way of serializing and materializing an object, for those that do not need more special way of doing it.!

FLObjectCluster subclass: #FLBitsObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLBitsObjectCluster commentStamp: 'MartinDias 8/1/2011 02:59' prior: 0!
I have the common behavior for generic storing and loading variable bits objects.!

!FLBitsObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/19/2011 13:24'!
add: anObject traceWith: anAnalyzer
	"Add an object to the cluster. We know the object doesn't have references."

	objects add: anObject! !

FLBitsObjectCluster subclass: #FLByteObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLByteObjectCluster commentStamp: 'MartinDias 8/1/2011 02:59' prior: 0!
I am the generic cluster for storing and loading variable byte objects.!

!FLByteObjectCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 20:20'!
materializeInstanceWith: aMaterializer

	| instance size |
	size := aMaterializer nextEncodedPositiveInteger.
	instance := theClass basicNew: size.
	aMaterializer stream next: size into: instance.
	^ instance
! !

!FLByteObjectCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 17:00'!
serializeInstance: anObject with: aSerializer

	aSerializer encodePositiveInteger: anObject basicSize.
	aSerializer stream nextBytesPutAll: anObject.! !

FLBitsObjectCluster subclass: #FLWordObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLWordObjectCluster commentStamp: 'MarianoMartinezPeck 9/6/2011 22:50' prior: 0!
I am the generic cluster for storing and loading variable word objects. Most word-like objects are subclasses from ArrayedCollection. Hence, we may have used the method #writeOn: to serialize, and #newFromStream: to materialize. 

#writeOn: is slow because it ALWAYS encode wors in a big endian format. Most machines today are even little endian. Even worst, #newFromStream: has to always do a #restoreEndianness. 

#newFromStream: is fast because it uses the method #nextWordsInto: which is a hack but that is really fast. The problem is that #nextWordsInto: has to always do the #restoreEndianness when we are in little endian machines (most of the time).

Hence, this cluster provives the best of both worlds: the serialization is done with a platform-dependen way, that is, the endianness will be the same of the machine where we are serializing. In addition, we serialize with the same hack that #nextWordsInto: uses, that is, the Bitmap >> hackBits:
And the materialization will use a variation of #nextWordsInto:  (#fuelNextWordsInto:) that ONLY does the #restoreEndianness if the machine where we are materializing has a different endianness than where we have serialized the object. To do this, while serializing, we store in the header of the Fuel stream, the endianness. Then in materialization, we compare agains that. 

Conclusion: we only restore endianness when needed.

Notice, in addition, that the method #newFromStream: sends the message #restoreEndianness. This method EXPECTS that the object was serialized in big endian format, and it will only change the endianness if we are materializing in a little endian machine. This means that if you serialize in little endian and materialize in big endian, you are screw.  For this reason, this cluster uses a method that always change the endianness, Bitmap class >> swapBytesIn:from:to: 

Future work: maybe we can modify and use the Bitmap hackBits:   not only for materialization but also for serialization. 
!

!FLWordObjectCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 20:20'!
materializeInstanceWith: aMaterializer

	| inst wideSize |

	wideSize :=  aMaterializer nextEncodedPositiveInteger.
	inst := theClass basicNew: wideSize.	

	aMaterializer stream fuelNextWordsInto: inst. 

	(aMaterializer isBigEndian = Smalltalk isBigEndian asBit)
		ifTrue: [^ inst ]
		ifFalse: [ ^ self swapBytesOf: inst ] 
	! !

!FLWordObjectCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 21:06'!
serializeInstance: aWordObject with: aSerializer

	aSerializer encodePositiveInteger: aWordObject basicSize.
	aSerializer stream nextWordsPut: aWordObject

! !

!FLWordObjectCluster methodsFor: 'endianness' stamp: 'MarianoMartinezPeck 9/6/2011 12:34'!
swapBytesOf: aWordObject
	Bitmap
                swapBytesIn: aWordObject
                from: 1
                to: aWordObject basicSize.
	^ aWordObject! !

!FLObjectCluster class methodsFor: 'instance creation' stamp: 'MartinDias 8/20/2011 06:54'!
for: aClass

	^ self basicNew
		initialize;
		theClass: aClass;
		yourself.! !

!FLObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 18:09'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket ! !

!FLObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 5/30/2011 03:45'!
clusterReferencesDo: aBlock

	aBlock value: theClass! !

!FLObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/29/2011 00:16'!
materializeInstancesStepWith: aMaterializer

	theClass := aMaterializer nextEncodedReference.
	super materializeInstancesStepWith: aMaterializer.! !

!FLObjectCluster methodsFor: 'printing' stamp: 'MartinDias 8/20/2011 23:10'!
printNameOn: aStream
	"Append to the argument, aStream, a sequence of characters that  
	identifies the receiver."
	
	super printNameOn: aStream.
	aStream nextPut: $[.
	self theClass printOn: aStream.
	aStream nextPut: $].
	! !

!FLObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/29/2011 00:16'!
serializeInstancesStepWith: aSerializer

	aSerializer encodeReferenceTo: theClass.
	super serializeInstancesStepWith: aSerializer.! !

!FLObjectCluster methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 03:42'!
theClass

	^theClass ! !

!FLObjectCluster methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 03:43'!
theClass: aClass

	theClass := aClass ! !

FLObjectCluster subclass: #FLPointerObjectCluster
	instanceVariableNames: 'variablesMapping'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLPointerObjectCluster commentStamp: 'MartinDias 8/1/2011 03:00' prior: 0!
I have the common behavior for storing and loading pointer objects.!

FLPointerObjectCluster subclass: #FLFixedObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLFixedObjectCluster commentStamp: 'MartinDias 5/30/2011 01:25' prior: 0!
I am a generic cluster for objects without indexable variables.!

!FLFixedObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 5/30/2011 03:42'!
materializeInstanceWith: aMaterializer

	^theClass basicNew! !

!FLFixedObjectCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/18/2011 14:53'!
serializeInstances: aSerializer! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/28/2011 23:57'!
afterMaterializationStepWith: aMaterializer  

	objects do: [ :anObject | anObject fuelAfterMaterialization ]! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 23:03'!
materializeInstancesStepWith: aMaterializer
	"Template method that let to the subclass the materialization using the received materializer"

	super materializeInstancesStepWith: aMaterializer.
		
	variablesMapping := FLVariablesMapping for: theClass.
	variablesMapping materializeMappingFrom: aMaterializer stream.! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 19:09'!
materializeReferencesOf: anObject with: aMaterializer

	variablesMapping materializeReferencesOf: anObject with: aMaterializer! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:56'!
materializeReferencesStepWith: aMaterializer

	objects do: [ :anObject | self materializeReferencesOf: anObject with: aMaterializer ]! !

!FLPointerObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/27/2011 22:32'!
referencesOf: anObject do: aBlock

	| ignoredInstanceVariableNames |
	ignoredInstanceVariableNames := theClass fuelIgnoredInstanceVariableNames.
	
	theClass fuelInstVarNamesAndOffsetsDo: [:name :index | 
		(ignoredInstanceVariableNames includes: name) 
			ifFalse: [ aBlock value: (anObject instVarAt: index) ]]
! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/29/2011 00:40'!
serializeInstancesStepWith: aSerializer

	super serializeInstancesStepWith: aSerializer.
	
	variablesMapping := FLVariablesMapping for: theClass.
	variablesMapping serializeMappingOn: aSerializer stream.! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 19:06'!
serializeReferencesOf: anObject with: aSerializer
	
	variablesMapping serializeReferencesOf: anObject with: aSerializer
! !

!FLPointerObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 22:55'!
serializeReferencesStepWith: aSerializer

	objects do: [ :anObject | self serializeReferencesOf: anObject with: aSerializer ]! !

FLPointerObjectCluster subclass: #FLVariableObjectCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLVariableObjectCluster commentStamp: 'MartinDias 5/30/2011 01:25' prior: 0!
I am a generic cluster for objects with indexable variables.!

FLVariableObjectCluster subclass: #FLMethodContextCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLMethodContextCluster commentStamp: 'MartinDias 5/30/2011 01:18' prior: 0!
I am a cluster for MethodContexts.!

!FLMethodContextCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 23:49'!
materializeInstanceWith: aMaterializer

	^ theClass newFromFrameSize: aMaterializer stream nextWord
! !

!FLMethodContextCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 21:37'!
serializeInstance: aMethodContext with: aSerializer

	aSerializer stream nextWordPut: aMethodContext method frameSize.
! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/1/2011 01:35'!
materializeInstanceWith: aMaterializer

	^theClass basicNew: aMaterializer nextEncodedReference! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 19:09'!
materializeReferencesOf: anObject with: aMaterializer

	super materializeReferencesOf: anObject with: aMaterializer.

	1 to: anObject basicSize 
		do: [ :index | anObject basicAt: index put: aMaterializer nextEncodedReference ]! !

!FLVariableObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/1/2011 01:36'!
referencesOf: anObject do: aBlock

	super referencesOf: anObject do: aBlock.

	aBlock value: anObject basicSize.

	1 to: anObject basicSize 
		do: [ :index | aBlock value: (anObject basicAt: index) ]! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/1/2011 01:35'!
serializeInstance: anObject with: aSerializer

	aSerializer encodeReferenceTo: anObject basicSize! !

!FLVariableObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/27/2011 19:06'!
serializeReferencesOf: anObject with: aSerializer

	super serializeReferencesOf: anObject with: aSerializer.
	
	1 to: anObject basicSize do: [ :index |
		aSerializer encodeReferenceTo: (anObject basicAt: index) ]! !

FLIteratingCluster subclass: #FLPointCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLPointCluster commentStamp: 'MarianoMartinezPeck 9/8/2011 22:33' prior: 0!
FLPointCluster is an optional class that optimizes Point instances, since there are a lot of instances in the system, it makes sense to optimize them. We take advantage of bytecode optimizations done by Pharo for messages like #@, #x and #y. In addition, we avoid the overhead of class reshape, etc. !

!FLPointCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/7/2011 14:18'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket! !

!FLPointCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/8/2011 21:35'!
materializeInstanceWith: aMaterializer
	"Since in Pharo #@ is associated with a special bytecode, it is faster than doing  Point basicNew"
	^ 0@0 
	! !

!FLPointCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/8/2011 21:45'!
materializeReferencesStepWith: aMaterializer

	objects do: [ :aPoint | 
		aPoint setX: aMaterializer nextEncodedReference setY: aMaterializer nextEncodedReference.	
	 ]! !

!FLPointCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 9/8/2011 21:37'!
referencesOf: aPoint do: aBlock

	aBlock value: aPoint x.
	aBlock value: aPoint y.
	! !

!FLPointCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/18/2011 13:58'!
serializeInstances: aSerializer! !

!FLPointCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/8/2011 21:44'!
serializeReferencesStepWith: aSerializer

	objects do: [ :aPoint | 
		aSerializer encodeReferenceTo: aPoint x.
		aSerializer encodeReferenceTo: aPoint y.
	]! !

FLIteratingCluster subclass: #FLPrimitiveCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLPrimitiveCluster commentStamp: 'MartinDias 8/29/2011 19:21' prior: 0!
I am a cluster that serializes and materializes interacting directly with the stream.!

FLPrimitiveCluster subclass: #FLBitmapCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLBitmapCluster commentStamp: 'MarianoMartinezPeck 9/6/2011 15:38' prior: 0!
FLBitmapCluster is an optional cluster that optimizes Bitmap serialization. Bitmap are word-like objects, hence, they could use FLWordObjectCluster.  This optimization takes advantange of the existance of methods in Bitmap that can compress and uncompress the bitmap. 

The method Bitmap >> writeOn:   uses the method #compressToByteArray  and  Bitmap class >> newFromStream:  uses the method #decompressFromByteArray:

  

!

!FLBitmapCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/16/2011 19:26'!
materializeFrom: aReadStream

	^Bitmap newFromStream: aReadStream! !

!FLBitmapCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/16/2011 19:26'!
serialize: aBitmap on: aWriteStream

	aBitmap writeOn: aWriteStream! !

FLPrimitiveCluster subclass: #FLByteStringCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLByteStringCluster commentStamp: 'MartinDias 5/30/2011 01:27' prior: 0!
I am a cluster for ByteString.!

!FLByteStringCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/15/2011 17:18'!
materializeFrom: aReadStream

	^ aReadStream nextString! !

!FLByteStringCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 8/25/2011 13:20'!
serialize: aSymbol on: aWriteStream

	aWriteStream nextStringPut: aSymbol! !

FLPrimitiveCluster subclass: #FLCharacterCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLCharacterCluster commentStamp: 'MartinDias 5/30/2011 01:27' prior: 0!
I am a cluster for Character.!

!FLCharacterCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 11:06'!
materializeFrom: aReadStream

	^Character value: aReadStream next! !

!FLCharacterCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/22/2010 17:33'!
serialize: aCharacter on: aWriteStream

	aWriteStream nextPut: aCharacter codePoint! !

FLPrimitiveCluster subclass: #FLDateAndTimeCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLDateAndTimeCluster commentStamp: 'MarianoMartinezPeck 9/2/2011 15:42' prior: 0!
FLDateAndTimeCluster is an optional cluster that optimizes the serialization of DateAndTime instances. This cluster can be removed and the regular serialization will still work.

We can do this cluster because we are sure that:

(DateAndTime allInstances collect: [:each | each julianDayNumber class]) asSet.  -> a Set(SmallInteger)
(DateAndTime allInstances collect: [:each | each nanoSecond class]) asSet. -> a Set(SmallInteger)
(DateAndTime allInstances collect: [:each | each secondsSinceMidnight class]) asSet. -> a Set(SmallInteger)
(DateAndTime allInstances collect: [:each | each offset class]) asSet. -> a Set(Duration)

Notice that we need to fix the subclasses of DateAndTime. In this case, TimeStamp. Otherwise, if subclasses add instance variables, they will answer #acceptFuel: of this class which follows a #visitDate. And that is not correct. Even more, FLDateAndTimeStamp directly uses the class DateAndTime to materialize.  !

!FLDateAndTimeCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 17:15'!
materializeFrom: aReadStream
	^ DateAndTime 
		fuelFrom: (aReadStream int32)
		nanoSecond: (aReadStream int32)
		seconds: (aReadStream int32)
		offset: (Duration seconds: (aReadStream int32) nanoSeconds: (aReadStream int32) ).
! !

!FLDateAndTimeCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 17:14'!
serialize: aDateAndTime on: aWriteStream
	aWriteStream 
		int32: aDateAndTime julianDayNumber;
		int32: aDateAndTime nanoSecond;
		int32: aDateAndTime secondsSinceMidnight;
		int32: aDateAndTime offset asSeconds;
		int32: aDateAndTime offset nanoSeconds.! !

FLPrimitiveCluster subclass: #FLDateCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLDateCluster commentStamp: 'MarianoMartinezPeck 9/2/2011 14:36' prior: 0!
FLDateCluster is an optional cluster that optimizes the serialization of Date instances. This cluster can be removed and the regular serialization will still work.

We can do this cluster because we are sure that:

(Date allInstances collect: [:each | each julianDayNumber class]) asSet.

answers only SmallInteger. 

And also because there are no subclasses. Otherwise, if subclasses add instance variables, they will answer #acceptFuel: of this class which follows a #visitDate. And that is not correct. !

!FLDateCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 17:15'!
materializeFrom: aReadStream
	
	^ Date fuelFromjulianDayNumber: (aReadStream int32)! !

!FLDateCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 17:14'!
serialize: aDate on: aWriteStream
	aWriteStream 
		int32: aDate julianDayNumber.! !

FLPrimitiveCluster subclass: #FLDurationCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLDurationCluster commentStamp: 'MarianoMartinezPeck 9/2/2011 14:36' prior: 0!
FLDurationCluster is an optional cluster that optimizes the serialization of Duration instances. This cluster can be removed and the regular serialization will still work.

We can do this cluster because we are sure that:

(Duration allInstances collect: [:each | each asSeconds class]) asSet.
(Duration allInstances collect: [:each | each nanoSeconds class]) asSet.

both answer only SmallInteger. 

And also because there are no subclasses. Otherwise, if subclasses add instance variables, they will answer #acceptFuel: of this class which follows a #visitDuration. And that is not correct. !

!FLDurationCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 17:15'!
materializeFrom: aReadStream
	^ Duration seconds: (aReadStream int32) nanoSeconds: (aReadStream int32) 
	! !

!FLDurationCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 17:14'!
serialize: aDuration on: aWriteStream
	aWriteStream 
		int32: aDuration asSeconds;
		int32: aDuration nanoSeconds
	! !

FLPrimitiveCluster subclass: #FLFloatCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLFloatCluster commentStamp: 'MarianoMartinezPeck 9/6/2011 18:13' prior: 0!
I am a cluster for 8-byte Floats. 

Implementation extracted from ReferenceStream.

Notice that this cluster is optional, since the current version of FLWordObjectCluster supports Floats as well. But be aware that it is not common that the regular way of serializing word objects will work also with floats. Float is special. Read Float >> basicAt:  and #basicAt:put:  for more details. 

Conclusion: it is optional because FLWordObjectCluster has a special way to serialize word objects which supports Float, but it may not be the case in other implementations. 
!

!FLFloatCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/21/2010 13:27'!
materializeFrom: aReadStream

	^ (Float new: 2)
		at: 1 put: (aReadStream nextNumber: 4);
		at: 2 put: (aReadStream nextNumber: 4);
		yourself.
! !

!FLFloatCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/21/2010 10:39'!
serialize: aFloat on: aWriteStream

	aWriteStream 
		nextNumber: 4 put: (aFloat at: 1);
		nextNumber: 4 put: (aFloat at: 2).! !

FLPrimitiveCluster subclass: #FLGlobalCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalCluster commentStamp: 'MartinDias 9/16/2011 14:49' prior: 0!
I am a cluster for objects that are reachables from Smalltalk global dictionary.!

FLGlobalCluster subclass: #FLGlobalAssociationCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalAssociationCluster commentStamp: 'MartinDias 5/30/2011 01:28' prior: 0!
I am a cluster for variable bindings that belong to Smalltalk global dictionary.

For example, 

	Smalltalk globals associationAt: #Transcript 
	
could be serialized and materialized by me.!

!FLGlobalAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
materializeFrom: aReadStream

	^self materializeGlobalAssociationFrom: aReadStream! !

!FLGlobalAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
serialize: anAssociation on: aWriteStream
	
	self serializeGlobalAssociationKeyed: anAssociation key on: aWriteStream ! !

FLGlobalCluster subclass: #FLGlobalClassCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalClassCluster commentStamp: 'MartinDias 5/30/2011 01:28' prior: 0!
I am a cluster for classes that should be present when materializing. In other words, it only serializes a reference to the class, using its name.!

!FLGlobalClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
materializeFrom: aReadStream

	^self materializeGlobalClassFrom: aReadStream ! !

!FLGlobalClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
serialize: aClass on: aWriteStream
	
	self serializeGlobalClass: aClass on: aWriteStream! !

FLGlobalCluster subclass: #FLGlobalClassSideCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalClassSideCluster commentStamp: 'MartinDias 8/1/2011 02:57' prior: 0!
I am a cluster for those Metaclasses and ClassTrait who are not stored in detail.!

!FLGlobalClassSideCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
materializeFrom: aReadStream

	^(self materializeGlobalClassFrom: aReadStream) classSide! !

!FLGlobalClassSideCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
serialize: aClassOrTraitClassSide on: aWriteStream

	self serializeGlobalClass: aClassOrTraitClassSide instanceSide on: aWriteStream
! !

!FLGlobalCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/15/2011 20:16'!
clusterBucketIn: aClusterization

	^aClusterization globalsBucket ! !

!FLGlobalCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
materializeGlobalAssociationFrom: aStream

	^Smalltalk globals 
		associationDeclareAt: aStream nextString asSymbol
! !

!FLGlobalCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
materializeGlobalClassFrom: aStream

	| className |
	className := aStream nextString asSymbol.
	^ Smalltalk globals 
		at: className
		ifAbsent: [ FLClassNotFound new signal: 'Class', className, ' not found at materialization' ]! !

!FLGlobalCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
serializeGlobalAssociationKeyed: aSymbol on: aStream
	
	aStream nextStringPut: aSymbol! !

!FLGlobalCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
serializeGlobalClass: aClass on: aStream
	
	aStream nextStringPut: aClass name ! !

FLGlobalCluster subclass: #FLGlobalValueCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLGlobalValueCluster commentStamp: 'MartinDias 5/30/2011 01:28' prior: 0!
I am a cluster for known objects that belong to Smalltalk global dictionary.

For example, Transcript could be serialized and materialized by me.!

!FLGlobalValueCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
materializeFrom: aReadStream

	^(self materializeGlobalAssociationFrom: aReadStream) value! !

!FLGlobalValueCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/19/2011 14:35'!
serialize: anObject on: aWriteStream
	
	self serializeGlobalAssociationKeyed: (Smalltalk keyAtValue: anObject) on: aWriteStream! !

!FLPrimitiveCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/19/2011 14:06'!
add: anObject traceWith: anAnalyzer
	"Add an object to the cluster. We know the object doesn't have references."

	objects addLast: anObject! !

!FLPrimitiveCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 00:28'!
clusterBucketIn: aClusterization

	^aClusterization primitivesBucket ! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:30'!
materializeFrom: aReadStream
	"Hook method that each subclass should customize for its special way of materializing"
	
	self subclassResponsibility ! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/29/2011 00:31'!
materializeInstanceWith: aMaterializer
	
	^self materializeFrom: aMaterializer stream.  ! !

!FLPrimitiveCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/19/2011 14:06'!
newSerializingCollection
	"Answer a collection for the objects that correspond to this cluster."

	^OrderedCollection new! !

!FLPrimitiveCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/19/2011 14:16'!
registerIndexesOn: aDictionary

	self flag: #todo. "converting objects collection here in this method is a bit confusing."
	objects := objects asIdentitySet.
	super registerIndexesOn: aDictionary.! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2010 13:29'!
serialize: anObject on: aWriteStream
	"Hook method that each subclass should customize for its special way of serializing"
	
	self subclassResponsibility! !

!FLPrimitiveCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/29/2011 00:24'!
serializeInstance: anObject with: aSerializer

	self serialize: anObject on: aSerializer stream! !

FLPrimitiveCluster subclass: #FLSmallIntegerCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLSmallIntegerCluster commentStamp: '<historical>' prior: 0!
I don't have so much sense as a class, but I know which of my subclasses corresponds to each SmallInteger.!

FLSmallIntegerCluster subclass: #FLSigned16BitsSmallIntegerCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLSigned16BitsSmallIntegerCluster commentStamp: 'MartinDias 5/30/2011 01:29' prior: 0!
I am a cluster for SmallInteger instances between 0 and 16rFFFF.!

!FLSigned16BitsSmallIntegerCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 22:19'!
materializeInstancesStepWith: aMaterializer

	| wordArray size |
	size := aMaterializer nextEncodedPositiveInteger.
	wordArray := ShortIntegerArray new: size.
	aMaterializer stream fuelNextWordsInto: wordArray.
	objects := OrderedCollection new: size.
	1 to: size do: [:index | objects add: (wordArray at: index)].
! !

!FLSigned16BitsSmallIntegerCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 21:25'!
serializeInstancesStepWith: aSerializer
	| wordArray size |

	size := objects size.
	wordArray := ShortIntegerArray new: size.
	objects doWithIndex: [ :instance :index| wordArray at: index put: instance ].
	aSerializer encodePositiveInteger: size.
	aSerializer stream nextWordsPut: wordArray.

! !

FLSmallIntegerCluster subclass: #FLSigned32BitsSmallIntegerCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLSigned32BitsSmallIntegerCluster commentStamp: 'MartinDias 5/30/2011 01:29' prior: 0!
I am a cluster for positive SmallInteger instances.!

!FLSigned32BitsSmallIntegerCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/19/2011 20:35'!
materializeInstancesStepWith: aMaterializer

	| wordArray size stream |
	size := aMaterializer nextEncodedPositiveInteger.
	stream := aMaterializer stream.
	
	(size < (1 << 16)) 
		ifTrue: [ objects := self newMaterializingCollection: size. 
			size timesRepeat: [objects add: stream int32] ]
		ifFalse: [
				wordArray := IntegerArray new: size.
				aMaterializer stream fuelNextWordsInto: wordArray.	
				objects :=	wordArray asOrderedCollection.
		]
	
! !

!FLSigned32BitsSmallIntegerCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/19/2011 20:34'!
serializeInstancesStepWith: aSerializer
	| wordArray size stream |
	stream := aSerializer stream.
	size := objects size.
	aSerializer encodePositiveInteger: size.
	(size < (1 << 16)) 
		ifTrue: [ objects do: [ :instance | stream int32: instance. ]]
		ifFalse: [
			wordArray := IntegerArray new: size.
			objects doWithIndex: [ :instance :index| wordArray at: index put: instance ].
			aSerializer stream nextWordsPut: wordArray.
		]
	! !

FLPrimitiveCluster subclass: #FLSymbolCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLSymbolCluster commentStamp: 'MartinDias 5/30/2011 01:29' prior: 0!
I am a cluster for Symbol instances.!

!FLSymbolCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/26/2010 14:22'!
materializeFrom: aReadStream

	^ aReadStream nextString asSymbol! !

!FLSymbolCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 8/25/2011 13:21'!
serialize: aSymbol on: aWriteStream

	aWriteStream nextStringPut: aSymbol! !

FLPrimitiveCluster subclass: #FLTimeCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLTimeCluster commentStamp: 'MarianoMartinezPeck 9/2/2011 14:36' prior: 0!
FLTimeCluster is an optional cluster that optimizes the serialization of Time instances. This cluster can be removed and the regular serialization will still work.

We can do this cluster because we are sure that:

(Time allInstances collect: [:each | each asSeconds class]) asSet.

answers only SmallInteger. 

And also because there are no subclasses. Otherwise, if subclasses add instance variables, they will answer #acceptFuel: of this class which follows a #visitTime. And that is not correct. !

!FLTimeCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 17:15'!
materializeFrom: aReadStream
	
	^ Time fromSeconds: (aReadStream int32)! !

!FLTimeCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 17:15'!
serialize: aTime on: aWriteStream
	aWriteStream 
		int32: aTime asSeconds.! !

FLIteratingCluster subclass: #FLRectangleCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters-Optionals'!

!FLRectangleCluster commentStamp: 'MarianoMartinezPeck 9/8/2011 22:53' prior: 0!
FLRectangleCluster is an optional class that optimizes Rectangle instances, since there are a lot of instances in the system, it makes sense to optimize them. We take advantage of bytecode optimizations done by Pharo for messages like #new, #@, #x and #y. In addition, we avoid the overhead of class reshape, etc. !

!FLRectangleCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 9/8/2011 22:43'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket! !

!FLRectangleCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/9/2011 18:15'!
materializeInstanceWith: aMaterializer
	"Since in Pharo #@ is associated with a special bytecode, it is faster than doing  Point basicNew"
	^ Rectangle basicNew! !

!FLRectangleCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/8/2011 22:51'!
materializeReferencesStepWith: aMaterializer

	objects do: [ :aRectangle | 
		aRectangle 
			setOrigin: aMaterializer nextEncodedReference @ aMaterializer nextEncodedReference	
		 	corner: aMaterializer nextEncodedReference @ aMaterializer nextEncodedReference.	
	 ]! !

!FLRectangleCluster methodsFor: 'analyzing' stamp: 'MarianoMartinezPeck 9/8/2011 22:45'!
referencesOf: aRectangle do: aBlock

	aBlock value: aRectangle origin x.
	aBlock value: aRectangle origin y.
	aBlock value: aRectangle corner x.
	aBlock value: aRectangle corner y.! !

!FLRectangleCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/18/2011 13:58'!
serializeInstances: aSerializer! !

!FLRectangleCluster methodsFor: 'serialize/materialize' stamp: 'MarianoMartinezPeck 9/8/2011 22:44'!
serializeReferencesStepWith: aSerializer

	objects do: [ :aRectangle | 
		aSerializer encodeReferenceTo: aRectangle origin x.
		aSerializer encodeReferenceTo: aRectangle origin y.
		aSerializer encodeReferenceTo: aRectangle corner x.
		aSerializer encodeReferenceTo: aRectangle corner y.
		
	]! !

FLCluster subclass: #FLSubstitutionCluster
	instanceVariableNames: 'substitutions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLSubstitutionCluster commentStamp: 'MartinDias 8/29/2011 19:25' prior: 0!
I am a cluster for objects that have to be replaced in the object graph by a substitute object.!

FLSubstitutionCluster subclass: #FLHookSubstitutionCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLHookSubstitutionCluster commentStamp: 'MartinDias 8/29/2011 19:30' prior: 0!
I am a cluster for substituted objects, where such substitution is provided by a method hook.

Suppose you want to substitute instances of WriteStream by nil. In such case, WriteStream has to implement these two methods:

fuelAccept: aVisitor
	^aVisitor visitSubstitution: self

fuelSubstitution
	^nil!

!FLHookSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:46'!
substitutionFor: anObject
	
	^anObject fuelSubstitution! !

FLSubstitutionCluster subclass: #FLPluggableSubstitutionCluster
	instanceVariableNames: 'factory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLPluggableSubstitutionCluster commentStamp: 'MartinDias 8/29/2011 19:32' prior: 0!
I am a cluster for substituted objects, where such substitution is provided by a factory block.

Suppose you want to substitute every integer in the graph by its string representation. In such case you should configure the analyzer through:

anAnalyzer when: [:x | x isInteger] substituteBy: [:x | x printString].

In this way, when you serialize #(1 2), you will then materialize #('1' '2')!

!FLPluggableSubstitutionCluster classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/29/2011 22:20'!
with: aFactory

	^self basicNew
		factory: aFactory;
		initialize;
		yourself.! !

!FLPluggableSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:46'!
factory
	^ factory! !

!FLPluggableSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:46'!
factory: anObject
	factory := anObject! !

!FLPluggableSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:45'!
substitutionFor: anObject
	
	^factory value: anObject! !

!FLSubstitutionCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:46'!
add: anObject traceWith: anAnalyzer

	substitutions 
		at: anObject 
		ifAbsent: [ 
			| substitution |
			substitution := self substitutionFor: anObject.
			substitutions at: anObject put: substitution.
			anAnalyzer trace: substitution ] ! !

!FLSubstitutionCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:50'!
clusterBucketIn: aClusterization

	^aClusterization substitutionsBucket! !

!FLSubstitutionCluster methodsFor: 'initialize-release' stamp: 'MartinDias 8/20/2011 23:17'!
initialize

	super initialize.
	substitutions := IdentityDictionary new.! !

!FLSubstitutionCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/29/2011 01:12'!
materializeInstancesStepWith: aMaterializer
	"This cluster does nothing."! !

!FLSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/29/2011 01:04'!
objects
	"This cluster does not have objects"
	
	^#()! !

!FLSubstitutionCluster methodsFor: 'printing' stamp: 'MartinDias 8/29/2011 01:16'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: '->'.
	substitutions printElementsOn: aStream! !

!FLSubstitutionCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/20/2011 23:18'!
registerIndexesOn: aDictionary

	substitutions keysAndValuesDo: [ :anObject :theSubstitution | 
		| substitutionIndex |
		substitutionIndex := aDictionary at: theSubstitution.
		aDictionary at: anObject put: substitutionIndex. ].! !

!FLSubstitutionCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/29/2011 01:12'!
serializeInstancesStepWith: aSerializer
	"This cluster does nothing."! !

!FLSubstitutionCluster methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 00:45'!
substitutionFor: anObject
	
	self subclassResponsibility! !

FLCluster subclass: #FLWellKnownObjectsCluster
	instanceVariableNames: ''
	classVariableNames: 'WellKnownObjects'
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLWellKnownObjectsCluster commentStamp: 'MartinDias 8/1/2011 03:00' prior: 0!
I am a cluster for some well known objects like nil, true and false. They have a special treatment because they are so common in the image.!

!FLWellKnownObjectsCluster classSide methodsFor: 'class initialization' stamp: 'MartinDias 8/30/2011 00:53'!
initialize

	WellKnownObjects := Array
		with: nil
		with: true
		with: false
		with: Smalltalk globals! !

!FLWellKnownObjectsCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/30/2011 00:48'!
add: anObject traceWith: anAnalyzer

	"Do nothing"! !

!FLWellKnownObjectsCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 00:28'!
clusterBucketIn: aClusterization

	^aClusterization primitivesBucket! !

!FLWellKnownObjectsCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/30/2011 00:44'!
materializeInstancesStepWith: aMaterializer
! !

!FLWellKnownObjectsCluster methodsFor: 'accessing' stamp: 'MartinDias 8/30/2011 00:53'!
objects

	^WellKnownObjects! !

!FLWellKnownObjectsCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/30/2011 00:46'!
registerIndexesOn: aDictionary

	self objects do: [ :instance | aDictionary at: instance put: aDictionary size + 1 ].! !

!FLWellKnownObjectsCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/30/2011 00:44'!
serializeInstancesStepWith: aSerializer
! !

Object subclass: #FLClusterization
	instanceVariableNames: 'baselevelBucket substitutionsBucket globalsBucket metalevelInstanceSideBucket metalevelClassSideBucket primitivesBucket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLClusterization commentStamp: 'MartinDias 8/29/2011 19:12' prior: 0!
I manage the clusters collected during the analysis step of serialization.!

!FLClusterization methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:16'!
add: aCluster traceWith: anAnalyzer

	| clusterBucket |
	clusterBucket := aCluster clusterBucketIn: self.
	
	clusterBucket 
		addIfNotPresent: aCluster 
		ifPresentDo: [ ^self ].

	aCluster clusterReferencesDo: [ :aChild | anAnalyzer trace: aChild ]! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:27'!
baselevelBucket
	
	^ baselevelBucket! !

!FLClusterization methodsFor: 'accessing' stamp: 'MartinDias 9/17/2011 00:29'!
clusterBuckets
	"Answer a new collection with all the clusters that have been collected in trace. Note that order is imporant."

	^ OrderedCollection new 
		add: globalsBucket;
		add: primitivesBucket;
		add: metalevelClassSideBucket;
		add: metalevelInstanceSideBucket;
		add: baselevelBucket;
		add: substitutionsBucket;
		yourself.! !

!FLClusterization methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 19:47'!
clusters
	"Returns collection with all the clustered objects, in Association instances."
	
	^self clusterBuckets inject: OrderedCollection new into: [:c :d | c addAll: d. c ].
! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 9/15/2011 20:16'!
globalsBucket
	
	^ globalsBucket ! !

!FLClusterization methodsFor: 'initialize-release' stamp: 'MartinDias 9/17/2011 00:29'!
initialize

	super initialize.
	primitivesBucket := IdentitySet new.
	baselevelBucket := IdentitySet new.
	globalsBucket := IdentitySet new.
	metalevelInstanceSideBucket := IdentitySet new.
	metalevelClassSideBucket := IdentitySet new.
	internalClassTraitsBucket := IdentitySet new.
	substitutionsBucket := IdentitySet new.
! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 9/17/2011 00:27'!
metalevelClassSideBucket
	
	^ metalevelClassSideBucket! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 9/17/2011 00:27'!
metalevelInstanceSideBucket
	
	^ metalevelInstanceSideBucket ! !

!FLClusterization methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 22:49'!
objectCount

	^self clusterBuckets sum: [ :aClusterBucket | 
		aClusterBucket
			inject: 0 
			into: [ :sum :aCluster | sum + aCluster objects size ] ].! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 9/17/2011 00:29'!
primitivesBucket
	
	^ primitivesBucket! !

!FLClusterization methodsFor: 'cluster buckets' stamp: 'MartinDias 8/20/2011 22:52'!
substitutionsBucket
	
	^ substitutionsBucket! !

Object subclass: #FLMapper
	instanceVariableNames: 'next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLMapper commentStamp: 'MartinDias 8/11/2011 03:09' prior: 0!
The purpose of my hierarchy is to map objects with clusters.!

FLMapper subclass: #FLClassPoolAssociationMapper
	instanceVariableNames: 'internalClasses classPoolBindings clustersByClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLClassPoolAssociationMapper commentStamp: 'MartinDias 5/30/2011 01:14' prior: 0!
I map variable class variable associations, i.e. bindings that belong to the class pool of a Class existent on Smalltalk globals. !

!FLClassPoolAssociationMapper methodsFor: 'accessing' stamp: 'MartinDias 8/18/2011 20:31'!
accepts: anObject

	^ anObject isVariableBinding and: [ (self isClassVariableBinding: anObject) and: [ self isExternal: anObject ] ]! !

!FLClassPoolAssociationMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 16:39'!
clusterFor: anObject
	
	| theClass |
	theClass := classPoolBindings at: anObject.
	
	^clustersByClass 
		at: theClass 
		ifAbsentPut: [ FLClassPoolAssociationCluster for: theClass ]! !

!FLClassPoolAssociationMapper methodsFor: 'initialize-release' stamp: 'MartinDias 8/18/2011 20:32'!
initialize

	super initialize.
	clustersByClass := IdentityDictionary new.
	internalClasses := OrderedCollection new.
	classPoolBindings := IdentityDictionary new.
	Smalltalk allClassesAndTraitsDo: [ :aClass | 
				aClass classPool associationsDo: [ :aBinding | 
					classPoolBindings at: aBinding put: aClass ] ].! !

!FLClassPoolAssociationMapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 7/6/2011 14:31'!
internalObjects: internalGlobalObjects

	internalClasses := internalGlobalObjects select: [ :anObject | anObject isBehavior ]! !

!FLClassPoolAssociationMapper methodsFor: 'private' stamp: 'eem 6/8/2011 11:50'!
isClassVariableBinding: anObject
	^classPoolBindings includesKey: anObject! !

!FLClassPoolAssociationMapper methodsFor: 'private' stamp: 'eem 6/8/2011 11:50'!
isExternal: anObject
	^ (internalClasses includes: (classPoolBindings at: anObject)) not! !

FLMapper subclass: #FLDefaultMapper
	instanceVariableNames: 'wellKnownObjectCluster characterCluster classCluster classTraitCluster compiledMethodCluster namedCompiledMethodCluster floatCluster metaclassCluster shortCluster symbolCluster traitCluster byteObjectClusters variableObjectClusters fixedObjectClusters wordObjectClusters byteStringCluster hookSubstitutionCluster dateCluster timeCluster durationCluster dateAndTimeCluster bitmapCluster pointCluster rectangleCluster smallIntegerCluster associationCluster methodDictionaryCluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLDefaultMapper commentStamp: 'MartinDias 8/7/2011 12:24' prior: 0!
I know how to map an object to its default cluster. I can map every object.!

!FLDefaultMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 12:22'!
accepts: anObject

	^ true! !

!FLDefaultMapper methodsFor: 'accessing' stamp: 'MartinDias 8/11/2011 03:02'!
chainedClusterFor: anObject

	^anObject fuelAccept: self! !

!FLDefaultMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 12:47'!
clusterFor: anObject

	^anObject fuelAccept: self
! !

!FLDefaultMapper methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 9/17/2011 14:36'!
initialize

	super initialize.
	fixedObjectClusters := IdentityDictionary new.
	variableObjectClusters := IdentityDictionary new.
	byteObjectClusters := IdentityDictionary new.
	wordObjectClusters := IdentityDictionary new.
	
	wellKnownObjectCluster := FLWellKnownObjectsCluster new.
	characterCluster := FLCharacterCluster new.
	classCluster := FLClassCluster new.
	classTraitCluster := FLClassTraitCluster new.
	compiledMethodCluster := FLCompiledMethodCluster new.
	namedCompiledMethodCluster := FLNamedPrimitiveMethodCluster new.
	floatCluster := FLFloatCluster new.
	metaclassCluster := FLMetaclassCluster new.
	shortCluster := FLSigned16BitsSmallIntegerCluster new.
	smallIntegerCluster := FLSigned32BitsSmallIntegerCluster new.
	symbolCluster := FLSymbolCluster new.
	traitCluster := FLTraitCluster new.
	byteStringCluster := FLByteStringCluster new.
	hookSubstitutionCluster := FLHookSubstitutionCluster new.
	dateCluster := FLDateCluster new.
	timeCluster := FLTimeCluster new. 
	durationCluster := FLDurationCluster new.
	dateAndTimeCluster := FLDateAndTimeCluster new.
	bitmapCluster := FLBitmapCluster new.
	pointCluster := FLPointCluster new.
	rectangleCluster := FLRectangleCluster new.
	associationCluster := FLAssociationCluster new.
	methodDictionaryCluster := FLMethodDictionaryCluster new.! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 9/9/2011 18:18'!
visitAssociation: anAssociation

	^associationCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 9/6/2011 15:09'!
visitBitmap: aBitmap

	^bitmapCluster
	! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:36'!
visitBoolean: aBoolean

	^wellKnownObjectCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/15/2011 17:20'!
visitByteString: aSymbol

	^byteStringCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:04'!
visitBytesObject: anObject

	^byteObjectClusters
		at: anObject class
		ifAbsentPut: [ FLByteObjectCluster for: anObject class ]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:06'!
visitCharacter: aCharacter

	^aCharacter isOctetCharacter 
		ifTrue: [ characterCluster ]
		ifFalse: [ 
			fixedObjectClusters
				at: aCharacter class
				ifAbsentPut: [ FLFixedObjectCluster for: aCharacter class ]]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:38'!
visitClass: aClass

	^classCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:38'!
visitClassTrait: aClassTrait

	^classTraitCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:39'!
visitCompiledMethod: aCompiledMethod

	^aCompiledMethod isNamedPrimitive
		ifTrue:  [ namedCompiledMethodCluster ]
		ifFalse: [ compiledMethodCluster ]
! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 9/1/2011 20:45'!
visitDate: aDate

	^dateCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 9/2/2011 15:15'!
visitDateAndTime: aDateAndTime

	^dateAndTimeCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 9/1/2011 23:20'!
visitDuration: aTime

	^durationCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:05'!
visitFixedObject: anObject

	^fixedObjectClusters
		at: anObject class
		ifAbsentPut: [ FLFixedObjectCluster for: anObject class ]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:39'!
visitFloat: aFloat

	^floatCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:05'!
visitLargeInteger: anObject

	^byteObjectClusters
		at: anObject class
		ifAbsentPut: [ FLByteObjectCluster for: anObject class ]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:39'!
visitMetaclass: aMetaclass

	^metaclassCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:05'!
visitMethodContext: aMethodContext

	^variableObjectClusters
		at: aMethodContext class
		ifAbsentPut: [ FLMethodContextCluster for: aMethodContext class ]
! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 9/17/2011 14:36'!
visitMethodDictionary: aMethodDictionary
	^ methodDictionaryCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 9/8/2011 21:31'!
visitPoint: aDate

	^pointCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 9/8/2011 22:47'!
visitRectangle: aRectangle

	^ rectangleCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 9/9/2011 20:58'!
visitSmallInteger: aSmallInteger

       (aSmallInteger < -16r8000) | (aSmallInteger >= 16r8000)
               ifTrue: [ ^ smallIntegerCluster]
               ifFalse: [ ^ shortCluster ]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/21/2011 01:08'!
visitSubstitution: anObject

	^hookSubstitutionCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:43'!
visitSymbol: aSymbol

	^symbolCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:43'!
visitSystemDictionary: anObject

	^wellKnownObjectCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 9/1/2011 21:09'!
visitTime: aTime

	^timeCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:43'!
visitTrait: aTrait

	^traitCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:43'!
visitUndefinedObject: anObject

	^wellKnownObjectCluster! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 9/2/2011 14:44'!
visitVariableObject: anObject

	^variableObjectClusters
		at: anObject class
		ifAbsentPut: [ FLVariableObjectCluster for: anObject class ]! !

!FLDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 03:07'!
visitWordsObject: anObject

	^wordObjectClusters
		at: anObject class
		ifAbsentPut: [ FLWordObjectCluster for: anObject class ]! !

FLMapper subclass: #FLGlobalAssociationMapper
	instanceVariableNames: 'internalAssociations cluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLGlobalAssociationMapper commentStamp: 'MartinDias 5/30/2011 01:15' prior: 0!
I map variable bindings that belong to Smalltalk global dictionary.

For example, 

	Smalltalk globals associationAt: #Transcript 
	
is mapped by me.!

!FLGlobalAssociationMapper methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 02:01'!
accepts: anObject

	^ anObject isVariableBinding and: [(self isGlobalAssociation: anObject) and: [ self isExternalAssociation: anObject ]].
! !

!FLGlobalAssociationMapper methodsFor: 'accessing' stamp: 'MartinDias 8/11/2011 02:34'!
clusterFor: anObject

	^cluster! !

!FLGlobalAssociationMapper methodsFor: 'initialize-release' stamp: 'MartinDias 8/11/2011 02:34'!
initialize

	super initialize.
	internalAssociations := OrderedCollection new.
	cluster := FLGlobalAssociationCluster new.! !

!FLGlobalAssociationMapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 7/6/2011 14:30'!
internalObjects: aCollection

	internalAssociations := aCollection collect: [:aGlobal | Smalltalk globals associationAt: aGlobal name ]! !

!FLGlobalAssociationMapper methodsFor: 'private' stamp: 'MartinDias 5/20/2011 04:10'!
isExternalAssociation: anObject

	^ (internalAssociations includes: anObject ) not! !

!FLGlobalAssociationMapper methodsFor: 'private' stamp: 'MartinDias 5/20/2011 04:13'!
isGlobalAssociation: anObject

	^ Smalltalk globals includesAssociation: anObject! !

FLMapper subclass: #FLGlobalBehaviorMapper
	instanceVariableNames: 'internalGlobals globalBehaviors classSideCluster instanceSideCluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLGlobalBehaviorMapper commentStamp: 'MartinDias 8/6/2011 23:51' prior: 0!
I map classes that belong to Smalltalk global dictionary.

For example, Integer is mapped by me.!

!FLGlobalBehaviorMapper methodsFor: 'accessing' stamp: 'MartinDias 8/6/2011 23:56'!
accepts: anObject

	^(anObject isBehavior or: [ anObject isTrait ]) 
		and: [(self isGlobal: anObject) and: [self isExternal: anObject ]]! !

!FLGlobalBehaviorMapper methodsFor: 'accessing' stamp: 'MartinDias 8/11/2011 02:35'!
clusterFor: anObject

	^anObject isClassSide
		ifTrue: [ classSideCluster ]
		ifFalse: [ instanceSideCluster ]
! !

!FLGlobalBehaviorMapper methodsFor: 'initialize-release' stamp: 'MartinDias 9/15/2011 19:51'!
initialize

	super initialize.
	internalGlobals := OrderedCollection new.
	globalBehaviors := IdentitySet new.
	Smalltalk globals allClassesAndTraitsDo: [:each | globalBehaviors add: each].
	classSideCluster := FLGlobalClassSideCluster new.
	instanceSideCluster := FLGlobalClassCluster new.! !

!FLGlobalBehaviorMapper methodsFor: 'accessing' stamp: 'MartinDias 8/6/2011 20:34'!
internalObjects: anObject
	internalGlobals := anObject! !

!FLGlobalBehaviorMapper methodsFor: 'private' stamp: 'MartinDias 8/6/2011 23:28'!
isExternal: anObject

	^(internalGlobals includes: anObject instanceSide) not! !

!FLGlobalBehaviorMapper methodsFor: 'private' stamp: 'MartinDias 8/6/2011 23:31'!
isGlobal: anObject

	^globalBehaviors includes: anObject instanceSide! !

FLMapper subclass: #FLGlobalMapper
	instanceVariableNames: 'cluster'
	classVariableNames: 'Globals'
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLGlobalMapper commentStamp: 'MartinDias 8/18/2011 20:46' prior: 0!
I map global objects that belong to Smalltalk dictionary.

For example, Transcript is mapped by me.!

!FLGlobalMapper classSide methodsFor: 'class initialization' stamp: 'MartinDias 8/18/2011 20:27'!
initialize

	self initializeGlobals ! !

!FLGlobalMapper classSide methodsFor: 'class initialization' stamp: 'MartinDias 8/18/2011 20:26'!
initializeGlobals

	Globals := IdentitySet new
		add: Display;
		add: World;
		add: Smalltalk;
		add: Transcript;
		add: SourceFiles;
		add: Processor;
		add: ImageImports;
		add: TextConstants;
		add: Undeclared;
		add: SystemOrganization;
		add: Sensor;
		add: ActiveWorld;
		add: ActiveHand;
		yourself! !

!FLGlobalMapper methodsFor: 'accessing' stamp: 'MartinDias 8/18/2011 20:29'!
accepts: anObject

	^Globals includes: anObject! !

!FLGlobalMapper methodsFor: 'accessing' stamp: 'MartinDias 8/11/2011 02:36'!
clusterFor: anObject

	^cluster! !

!FLGlobalMapper methodsFor: 'initialize-release' stamp: 'MartinDias 9/15/2011 19:51'!
initialize

	super initialize.
	cluster := FLGlobalValueCluster new.
		! !

FLMapper subclass: #FLLightGlobalMapper
	instanceVariableNames: 'classSideCluster instanceSideCluster globalCluster'
	classVariableNames: 'Globals'
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLLightGlobalMapper commentStamp: 'MartinDias 8/20/2011 23:43' prior: 0!
I map classes, traits, and global objects that belong to Smalltalk dictionary. I am included in FLAnalyzer newLight behavior. 

I am a fast simplified replace to this four clusters included in FLAnaluzer newFull behavior: 
- FLGlobalMapper
- FLGlobalBehaviorMapper
- FLClassPoolAssociationMapper
- FLGlobalAssociationMapper 

For example, the global Transcript as well as any class in the image, are mapped by me.!

!FLLightGlobalMapper class methodsFor: 'class initialization' stamp: 'MartinDias 8/18/2011 20:12'!
initialize

	self initializeGlobals! !

!FLLightGlobalMapper class methodsFor: 'class initialization' stamp: 'MartinDias 8/18/2011 20:20'!
initializeGlobals

	Globals := IdentitySet new
		add: Display;
		add: World;
		add: Smalltalk;
		add: Transcript;
		add: SourceFiles;
		add: Processor;
		add: ImageImports;
		add: TextConstants;
		add: Undeclared;
		add: SystemOrganization;
		add: Sensor;
		add: ActiveWorld;
		add: ActiveHand;
		yourself! !

!FLLightGlobalMapper methodsFor: 'accessing' stamp: 'MartinDias 8/18/2011 20:19'!
chainedClusterFor: anObject
	"Uses the Chain of Responsibility pattern to answer the cluster which maps with the received object."

	(anObject isBehavior or: [ anObject isTrait ]) ifTrue: [
		^anObject isClassSide
			ifTrue: [ classSideCluster ]
			ifFalse: [ instanceSideCluster ]].

	(Globals includes: anObject) ifTrue: [ ^globalCluster ].
	
	^next chainedClusterFor: anObject.! !

!FLLightGlobalMapper methodsFor: 'initialize-release' stamp: 'MartinDias 9/15/2011 19:51'!
initialize

	super initialize.
	classSideCluster := FLGlobalClassSideCluster new.
	instanceSideCluster := FLGlobalClassCluster new.
	globalCluster := FLGlobalValueCluster new.! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 02:01'!
accepts: anObject
	"Tells if the received object is handled by this analyzer"

	self subclassResponsibility ! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 20:21'!
chainedClusterFor: anObject
	"Uses the Chain of Responsibility pattern to answer the cluster which maps with the received object."

	^(self accepts: anObject)
		ifTrue: [ self clusterFor: anObject ]
		ifFalse: [ next chainedClusterFor: anObject ].
! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 20:19'!
clusterFor: anObject
	"Answers the cluster which maps with the received object."

	self subclassResponsibility ! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/6/2011 20:37'!
internalObjects: aCollection! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 17:19'!
next
	^ next! !

!FLMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 17:19'!
next: anObject
	next := anObject! !

FLMapper subclass: #FLPluggableSubstitutionMapper
	instanceVariableNames: 'condition substitutionCluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Mappers'!

!FLPluggableSubstitutionMapper commentStamp: 'MartinDias 8/20/2011 23:45' prior: 0!
I map substituted objects. 

For example, suppose you want to substitute instances of WriteStream by nil. In such case, I will map every WriteStream instance to my substitutions cluster, and its factory block will value to nil with any argument.
!

!FLPluggableSubstitutionMapper class methodsFor: 'instance creation' stamp: 'MartinDias 8/29/2011 22:20'!
when: aCondition substituteBy: aFactory

	^self basicNew
		condition: aCondition;
		substitutionCluster: (FLPluggableSubstitutionCluster with: aFactory);
		initialize;
		yourself! !

!FLPluggableSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 19:37'!
accepts: anObject

	^condition value: anObject! !

!FLPluggableSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 20:31'!
clusterFor: anObject

	^substitutionCluster! !

!FLPluggableSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 19:37'!
condition
	^ condition! !

!FLPluggableSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 19:37'!
condition: anObject
	condition := anObject! !

!FLPluggableSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 20:33'!
substitutionCluster
	^ substitutionCluster! !

!FLPluggableSubstitutionMapper methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 20:33'!
substitutionCluster: anObject
	substitutionCluster := anObject! !

Object subclass: #FLMaterialization
	instanceVariableNames: 'stream clusterCount indexCluster clusters root objects isBigEndian'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLMaterialization class methodsFor: 'instance creation' stamp: 'MartinDias 9/10/2011 18:32'!
on: aReadStream

	^self basicNew
		initializeWith: aReadStream;
		yourself
! !

!FLMaterialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:58'!
afterMaterializationStep

	clusters do: [ :aCluster | aCluster afterMaterializationStepWith: self ]! !

!FLMaterialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:58'!
clusterCount

	^clusterCount ! !

!FLMaterialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:58'!
clusterInstancesStep

	| aCluster |
	aCluster := self nextEncodedCluster.
	aCluster materializeInstancesStepWith: self.
	self registerAll: aCluster objects.
	aCluster materializePostInstancesStepWith: self.
	clusters add: aCluster.! !

!FLMaterialization methodsFor: 'private' stamp: 'MarianoMartinezPeck 9/15/2011 12:36'!
headerStep

	objects := OrderedCollection new: self nextEncodedPositiveInteger.
	objects resetTo: 1. "Hack that avoids OrderedCollection>>makeRoomAtLast. In fact, this has already been fixed in Pharo 1.4: http://code.google.com/p/pharo/issues/detail?id=1686"

	clusterCount := self nextEncodedPositiveInteger.
	clusters := OrderedCollection new: clusterCount.
	clusters resetTo: 1. "Hack that avoids OrderedCollection>>makeRoomAtLast"
	
	isBigEndian := self nextEncodedPositiveInteger.
! !

!FLMaterialization methodsFor: 'initialize-release' stamp: 'MartinDias 9/10/2011 18:33'!
initializeWith: aReadStream

	self initialize.
	stream := aReadStream.! !

!FLMaterialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:58'!
instancesStep

	clusterCount timesRepeat: [ self clusterInstancesStep ]
! !

!FLMaterialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:58'!
isBigEndian
	^ isBigEndian ! !

!FLMaterialization methodsFor: 'decoding' stamp: 'MartinDias 9/9/2011 01:58'!
nextEncodedCluster

	^(Smalltalk classNamed: stream nextString) new! !

!FLMaterialization methodsFor: 'decoding' stamp: 'MartinDias 9/9/2011 01:58'!
nextEncodedInteger
	
	^FLPositiveSmallIntegerCluster new materializeFrom: stream! !

!FLMaterialization methodsFor: 'decoding' stamp: 'MartinDias 9/11/2011 19:33'!
nextEncodedPositiveInteger
	
	^ stream uint32! !

!FLMaterialization methodsFor: 'decoding' stamp: 'MartinDias 9/11/2011 19:33'!
nextEncodedReference 

	^ objects at: (stream uint32)! !

!FLMaterialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:58'!
objects
	"Answer a collection with the materialized objects."

	^ objects ! !

!FLMaterialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:58'!
referencesStep

	clusters do: [ :aCluster | aCluster materializeReferencesStepWith: self ]! !

!FLMaterialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:58'!
registerAll: materializedObjects
	
	objects addAll: materializedObjects ! !

!FLMaterialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:58'!
root

	^ root ! !

!FLMaterialization methodsFor: 'materializing' stamp: 'MartinDias 9/10/2011 18:17'!
run

	self headerStep.
	self instancesStep.
	self referencesStep.
	self trailerStep.
	self afterMaterializationStep.! !

!FLMaterialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:58'!
stream
	^ stream! !

!FLMaterialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:58'!
trailerStep

	root := self nextEncodedReference! !

Object subclass: #FLMaterializer
	instanceVariableNames: 'materializationFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLMaterializer commentStamp: 'MartinDias 8/29/2011 19:06' prior: 0!
I am a binary object materializer. 

See an example of use in FLSerializer's documentation.

!

!FLMaterializer class methodsFor: 'materializing' stamp: 'MartinDias 9/9/2011 20:25'!
materializeFromByteArray: byteArray

	^(self newDefault materializeFrom: byteArray readStream) root! !

!FLMaterializer class methodsFor: 'materializing' stamp: 'MartinDias 9/9/2011 20:24'!
materializeFromFileNamed: aFilename

	^StandardFileStream 
		oldFileNamed: aFilename 
		do: [:aFileStream | 
			(self newDefault materializeFrom: aFileStream binary) root ]
! !

!FLMaterializer class methodsFor: 'instance creation' stamp: 'MartinDias 9/10/2011 18:16'!
newDefault

	^self new! !

!FLMaterializer methodsFor: 'initialize-release' stamp: 'MartinDias 9/13/2011 19:19'!
initialize

	super initialize.
	self setDefaultMaterialization.! !

!FLMaterializer methodsFor: 'materializing' stamp: 'MartinDias 9/14/2011 00:17'!
materializeFrom: aStream

	^materializationFactory value: aStream! !

!FLMaterializer methodsFor: 'protected' stamp: 'MartinDias 9/14/2011 00:23'!
setDefaultMaterialization

	materializationFactory := [:aStream | (FLMaterialization on: aStream) run; yourself ]
	
	
! !

Object subclass: #FLSerialization
	instanceVariableNames: 'stream indexCluster root objectsIndexes clusters objectCount analyzer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLSerialization class methodsFor: 'instance creation' stamp: 'MartinDias 9/10/2011 18:06'!
root: anObject on: aWriteStream analyzer: anAnalyzer
	
	^ self basicNew
		initializeRoot: anObject
		on: aWriteStream
		analyzer: anAnalyzer;
		yourself! !

!FLSerialization methodsFor: 'accessing' stamp: 'MartinDias 9/10/2011 17:37'!
analysis
	^ analyzer! !

!FLSerialization methodsFor: 'private' stamp: 'MartinDias 9/11/2011 19:52'!
analysisStep

	| anAnalysis |
	anAnalysis := analyzer analysisFor: root.
	clusters := anAnalysis clusterization clusters.
	objectCount := anAnalysis clusterization objectCount.
	objectsIndexes := IdentityDictionary new: objectCount.
! !

!FLSerialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:11'!
clusterInstancesStepOf: aCluster

	self encodeIdOf: aCluster.
	aCluster serializeInstancesStepWith: self.
	aCluster serializePostInstancesStepWith: self.
! !

!FLSerialization methodsFor: 'accessing' stamp: 'MartinDias 9/13/2011 16:34'!
clusters
	^ clusters! !

!FLSerialization methodsFor: 'encoding' stamp: 'MartinDias 9/9/2011 01:11'!
encodeIdOf: aCluster

	stream nextStringPut: aCluster class name! !

!FLSerialization methodsFor: 'encoding' stamp: 'MartinDias 9/9/2011 01:11'!
encodeInteger: anInteger

	FLPositiveSmallIntegerCluster new 
		serialize: anInteger
		on: stream! !

!FLSerialization methodsFor: 'encoding' stamp: 'MartinDias 9/11/2011 19:33'!
encodePositiveInteger: anInteger

	stream uint32: anInteger! !

!FLSerialization methodsFor: 'encoding' stamp: 'MartinDias 9/11/2011 19:33'!
encodeReferenceTo: anObject

	self encodePositiveInteger: (objectsIndexes at: anObject)! !

!FLSerialization methodsFor: 'private' stamp: 'MartinDias 9/11/2011 19:33'!
headerStep

	self encodePositiveInteger: objectCount.
	self encodePositiveInteger: clusters size.
	self encodePositiveInteger: Smalltalk isBigEndian asBit.! !

!FLSerialization methodsFor: 'initialize-release' stamp: 'MartinDias 9/12/2011 00:12'!
initializeRoot: anObject on: aWriteStream analyzer: anAnalyzer
	
	self initialize.
	root := anObject.
	stream := aWriteStream.
	analyzer := anAnalyzer.! !

!FLSerialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:11'!
instancesStep

	clusters do: [ :aCluster | aCluster registerIndexesOn: objectsIndexes ].
	clusters do: [ :aCluster | self clusterInstancesStepOf: aCluster ]! !

!FLSerialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:11'!
referencesStep

	clusters do: [ :aCluster | aCluster serializeReferencesStepWith: self ]! !

!FLSerialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:33'!
root
	^ root! !

!FLSerialization methodsFor: 'serializing' stamp: 'MartinDias 9/9/2011 02:05'!
run
	"Serialize the graph starting at the root object."

	self analysisStep.
	self headerStep.
	self instancesStep.
	self referencesStep.
	self trailerStep.

	stream flush. "This is necessary when using our buffered stream."! !

!FLSerialization methodsFor: 'accessing' stamp: 'MartinDias 9/9/2011 01:11'!
stream
	^ stream! !

!FLSerialization methodsFor: 'private' stamp: 'MartinDias 9/9/2011 01:11'!
trailerStep

	self encodeReferenceTo: root! !

Object subclass: #FLSerializer
	instanceVariableNames: 'analyzer streamFactory serializationFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLSerializer commentStamp: 'MartinDias 8/29/2011 19:10' prior: 0!
I am a binary object serializer.

An example of use:

| sourceArray loadedArray |
sourceArray := 
	Array 
		with: 'a string' 
		with: Transcript
		with: [ Transcript show: 'a string' ].

"Store to the file"
FLSerializer serialize: sourceArray toFileNamed: 'example.FL'. 

"Load from the file"
loadedArray := FLMaterializer materializeFromFileNamed: 'example.FL'. 

"The arrays are not the same"
[ sourceArray ~~ loadedArray ] assert.

"The strings are not the same"
[ sourceArray first ~~ loadedArray first ] assert.
[ sourceArray first = loadedArray first ] assert.

"The global instance Transcript is the same"
[ sourceArray second == loadedArray second ] assert.

"Appreciate in Transcript that the loaded block prints a string"
loadedArray third value.
!

!FLSerializer class methodsFor: 'instance creation' stamp: 'MartinDias 9/9/2011 02:19'!
newDefault

	^self new! !

!FLSerializer class methodsFor: 'serializing' stamp: 'MartinDias 9/9/2011 18:25'!
serialize: anObject toFileNamed: aFilename

	StandardFileStream 
		fileNamed: aFilename 
		do: [ :aFileStream | 
			self newDefault serialize: anObject on: aFileStream binary ]! !

!FLSerializer class methodsFor: 'serializing' stamp: 'MartinDias 9/9/2011 17:03'!
serializeInMemory: anObject
	"No stream is needed by the user. An internal in-memory stream will be used. This method returns a ByteArray representing the serialization"
	
	| aStream |
	aStream := (ByteArray new: 100) writeStream.
	
	self newDefault serialize: anObject on: aStream.
	
	^aStream contents! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 17:33'!
analyzer
	^ analyzer! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 8/20/2011 17:33'!
analyzer: anObject
	analyzer := anObject! !

!FLSerializer methodsFor: 'protected' stamp: 'MartinDias 9/13/2011 19:26'!
defaultAnalyzer

	^FLAnalyzer newDefault! !

!FLSerializer methodsFor: 'initialize-release' stamp: 'MartinDias 9/13/2011 19:26'!
initialize

	super initialize.
	self setDefaultSerialization.
	self writeBuffered.
	analyzer := self defaultAnalyzer.
! !

!FLSerializer methodsFor: 'serializing' stamp: 'MartinDias 9/14/2011 00:17'!
serialize: anObject on: aStream 
	"Serialize the graph starting at the root object received."

	^serializationFactory value: anObject value: aStream
! !

!FLSerializer methodsFor: 'protected' stamp: 'MartinDias 9/14/2011 00:22'!
setDefaultSerialization

	serializationFactory := [:anObject :aStream | 
		(FLSerialization 
			root: anObject 
			on: (streamFactory value: aStream) 
			analyzer: analyzer)
			run;
			yourself ]! !

!FLSerializer methodsFor: 'configuring-stream' stamp: 'MartinDias 9/9/2011 20:18'!
writeBuffered

	self writeBufferedSize: 4096  ! !

!FLSerializer methodsFor: 'configuring-stream' stamp: 'MartinDias 9/9/2011 20:18'!
writeBufferedSize: aSize

	streamFactory := [:aStream | 
		(FLBufferedWriteStream on: aStream)
			sizeBuffer: aSize;
			yourself ]! !

!FLSerializer methodsFor: 'configuring-stream' stamp: 'MartinDias 9/9/2011 20:18'!
writeDirectly

	streamFactory := [:aStream | aStream ]! !

Object subclass: #FLSimpleStack
	instanceVariableNames: 'array slotIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Collections'!

!FLSimpleStack commentStamp: 'MarianoMartinezPeck 6/5/2011 12:29' prior: 0!
FLSimpleStack is a simple stack used in Fuel to avoid a recursion while traversing the graph to serialize. While analizing the content of an object (inst vars) we can encounter simple objects (direct string representation) or complex objects (composite). In the latter case we start analizing those instVars.

So... Imagine object X with two instVars Y and Z. Imagine the method of the traverse is called #analye:.  So you do #analize: X.  Then, in such method you check whether X has regular pointers to regular objects, and if true, you analize them. So in this case you would send #analyze: Y and #analize: Z, generating the recursion. Right ?  Notice that the time between analyse: Y and analyse: Z is dependent on the graph that is attached to Y. Y can have objects that have further objects etc. So leaving X aside, Y can be the first object to serialize and Z the last.

With this FLSimpleStack, instead of sending #analyze: Y and #analize: Z  what Fuel does is just to do a push on a stack:   #push: Y  and #push: Z. And then we are done with X, so we pop from the stack and we continue with the next object (at some point in the future we will pop Y and Z) When there are no more objects in the stack it means we are done. !

!FLSimpleStack class methodsFor: 'instance creation' stamp: 'MartinDias 5/15/2011 23:11'!
hintSize

	^64! !

!FLSimpleStack class methodsFor: 'instance creation' stamp: 'MartinDias 5/15/2011 23:11'!
new
	
	^ self new: self hintSize! !

!FLSimpleStack class methodsFor: 'instance creation' stamp: 'MartinDias 5/15/2011 23:09'!
new: nElements
	"Create a Set large enough to hold nElements without growing"
	^ self basicNew initialize: nElements! !

!FLSimpleStack methodsFor: 'accessing' stamp: 'MartinDias 5/15/2011 23:08'!
capacity

	^ array size! !

!FLSimpleStack methodsFor: 'adding' stamp: 'MartinDias 5/15/2011 23:15'!
grow
	"Become larger in capacity."
	| newArray |
	newArray := Array new: array size * 2.
	newArray replaceFrom: 1 to: array size with: array startingAt: 1.
	array := newArray! !

!FLSimpleStack methodsFor: 'initialize-release' stamp: 'MartinDias 5/15/2011 23:10'!
initialize: hintSize

	array := Array new: hintSize.
	slotIndex := 1.! !

!FLSimpleStack methodsFor: 'accessing' stamp: 'MartinDias 5/15/2011 22:54'!
isEmpty

	^ 1 = slotIndex! !

!FLSimpleStack methodsFor: 'removing' stamp: 'MartinDias 5/15/2011 22:52'!
pop
	"Returns the first element and remove it from the stack."

	slotIndex := slotIndex - 1.	
	^ array at: slotIndex ! !

!FLSimpleStack methodsFor: 'printing' stamp: 'MartinDias 6/3/2011 20:25'!
printOn: aStream 
	"Append a sequence of characters that identify the receiver to aStream."

	super printOn: aStream.
	array printElementsOn: aStream
! !

!FLSimpleStack methodsFor: 'adding' stamp: 'MartinDias 5/15/2011 22:44'!
push: anObject 
	"Adds a new object of any kind on top of the stack."

	array at: slotIndex put: anObject.
	array size = slotIndex ifTrue: [ self grow ].
	slotIndex := slotIndex + 1.
	^ anObject.! !

Object subclass: #FLVariablesMapping
	instanceVariableNames: 'mapping theClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Clusters'!

!FLVariablesMapping commentStamp: 'MartinDias 8/1/2011 03:01' prior: 0!
I am used to materialize instance variables in an object, tolerating "class shape changing".

Cases tolerated are:
- instance variable added
- instance variable order change
- instance variable removed
!

!FLVariablesMapping class methodsFor: 'instance creation' stamp: 'MartinDias 7/27/2011 20:29'!
for: aClass
	
	^ self basicNew
		theClass: aClass;
		yourself.! !

!FLVariablesMapping methodsFor: 'private' stamp: 'MartinDias 7/27/2011 23:21'!
instanceVariableNamesToSerialize

	| ignoredInstanceVariableNames instanceVariableNamesToSerialize |
	ignoredInstanceVariableNames := theClass fuelIgnoredInstanceVariableNames.
	instanceVariableNamesToSerialize := OrderedCollection new.

	theClass fuelInstVarNamesAndOffsetsDo: [:name :index | 
		(ignoredInstanceVariableNames includes: name) 
			ifFalse: [ instanceVariableNamesToSerialize add: name ]].

	^ instanceVariableNamesToSerialize! !

!FLVariablesMapping methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 23:22'!
materializeMappingFrom: aStream

	| instanceVariableNamesSerialized |
	instanceVariableNamesSerialized := (1 to: aStream next) collect: [ :index | aStream nextString ].

	mapping := instanceVariableNamesSerialized  
		collect: [ :name | theClass instVarIndexFor: name ifAbsent: [ nil ] ].

! !

!FLVariablesMapping methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 20:37'!
materializeReferencesOf: anObject with: aMaterializer 

	mapping do: [ :index | 
			| reference |
			reference := aMaterializer nextEncodedReference.
			index ifNotNil: [
				anObject 
					instVarAt: index
					put: reference ]].! !

!FLVariablesMapping methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 23:22'!
serializeMappingOn: aStream

	| instanceVariableNamesToSerialize |
	instanceVariableNamesToSerialize := self instanceVariableNamesToSerialize.

	aStream nextPut: instanceVariableNamesToSerialize size.	
	instanceVariableNamesToSerialize do: [ :name | aStream nextStringPut: name ].

	mapping := instanceVariableNamesToSerialize 
		collect: [ :name | theClass instVarIndexFor: name ].! !

!FLVariablesMapping methodsFor: 'serialize/materialize' stamp: 'MartinDias 7/27/2011 20:02'!
serializeReferencesOf: anObject with: aSerializer 

	mapping do: [ :index | 
		aSerializer encodeReferenceTo: (anObject instVarAt: index) ].! !

!FLVariablesMapping methodsFor: 'accessing' stamp: 'MartinDias 7/27/2011 20:30'!
theClass
	^ theClass! !

!FLVariablesMapping methodsFor: 'accessing' stamp: 'MartinDias 7/27/2011 20:30'!
theClass: anObject
	theClass := anObject! !

Object subclass: #FuelUtilities
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Utilities'!

!FuelUtilities commentStamp: 'MarianoMartinezPeck 6/5/2011 12:25' prior: 0!
FuelUtilities is a class that contains utilities when developing Fuel. Right now the only usage is to do some cleanings. Sometimes updating versions from Monticello bring problems with traits or things like that. Right now the only usage is:

FuelUtilities cleanUpEverything.
!

!FuelUtilities classSide methodsFor: 'cleaning' stamp: 'MartinDias 8/11/2011 02:46'!
cleanUpEverything
	"Sometimes when updating Fuel versions Monticello gets stupid and we need to do some cleanups."
	Smalltalk garbageCollect.
	Smalltalk fixObsoleteReferences.

	(PackageInfo named: 'Fuel') classes,
	(PackageInfo named: 'FuelFameExtension') classes,
	(PackageInfo named: 'FuelContainer') classes,
	(PackageInfo named: 'FuelMooseExtension') classes
	    do: [:c | c compileAll ].
	((PackageInfo named: 'Fuel') classes select: [:each | each isBehavior ]) do: [:each | each initialize]
! !

!Object methodsFor: '*Fuel' stamp: 'MartinDias 9/13/2011 19:52'!
asIdentitySet
	"Answer an IdentitySet with the receiver as its only element."

	^ IdentitySet with: self! !

!Object methodsFor: '*Fuel' stamp: 'MartinDias 8/6/2011 19:43'!
fuelAccept: aVisitor

	self class isFixed
		ifTrue: [ ^aVisitor visitFixedObject: self ].
	self class isPointers
		ifTrue: [ ^aVisitor visitVariableObject: self ].
	self class isBytes
		ifTrue: [ ^aVisitor visitBytesObject: self ].
	self class isWords
		ifTrue: [	 ^aVisitor visitWordsObject: self ].
	self error: 'Something is wrong!!'! !

!Object methodsFor: '*Fuel' stamp: 'MartinDias 8/27/2011 19:03'!
fuelAfterMaterialization
	"Materialization process will send this message after materializing instances of my class"! !
FLLightGlobalMapper initialize!
FLGlobalMapper initialize!
FLWellKnownObjectsCluster initialize!
